#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Dec  1 16:13:37 2020

@author: Andreas Rauchöcker

This script provides paths and functions to load data of different sorts for
the publication "Cold-Air Pool Development in a small Alpine Valley" in QJRMS.
"""

import numpy as np
import pandas as pd
import salem
from scipy.io import loadmat
from wrf import rh, td
from plotting_routines import loc_coords, SFC_EBalance, theta_to_temp, WS2wrf, rh_to_mr



#%% paths
datapath = r'../data_measured/'
mnames = ['MOMAA02.dat', 'MOMAA03.dat', 'MOMAA04.dat',
             'MOMAA07.dat', 'MOMAA08.dat', 'MOMAA10.dat']
# path to TAWES station data
spath = datapath + r'TAWES_seefeld.txt'
# paths to radiation data
rad_paths = [r'MOMAA04_rad.dat',
             r'MOMAA08_rad.dat']
# path to turbulent flux data
turb_paths = [datapath + r'MOMAA04_turb.csv',
              datapath + r'MOMAA08_turb.csv']
# path to pseudoprofile data
pseudopath = datapath + r'PseudoProfile_basin.mat'

# timestamp of data to process
times = pd.date_range(start='1/16/2020 12:00:00', end='1/17/2020 12:00:00', freq='10min')
times_maps = pd.date_range(start='1/16/2020 16:00:00', end='1/17/2020 06:10:00', freq='2h')
times_isonde = pd.to_datetime(pd.DataFrame({'year': 2020, 
                                        'month': 1, 
                                        'day': [13], 
                                        'hour': [3]}))
bound_times = pd.to_datetime(pd.DataFrame({'year': 2020, 
                                            'month': 1, 
                                            'day': [16, 17], 
                                            'hour': [12, 12],
                                            'minute': [0, 0]}))

#%% FUNCTIONS
def load_data(dspath, central, lon_view, lat_view, times, sondes=[],
              full_data=True):
    '''
    load WRF output as salem dataset and subsest to region of interest
    
    Parameters
    ----------
    dspath : str
        path to WRF output file.
    central : list/tuple
        coordinates of central point for subsetting dataset
    lon_view : int
        longitude range around central to look at.
    lat_view : int
        lattitude range around central to look at.
    times : list/array
        list of times to be analyzed.
    sondes : list
        list of coordinates for which profiles should be retrieved
    full_data : bool
        if False, output of full data as empty dataset. Default is True.

    Returns
    -------
    ds_sub : salem dataset
        subset of wrfoutput
    sondes_list : list of salem datasets
        list with dataset of profiles
    cloc : list/tuple
        nearest grid point coordinate to central grid location
    '''
    print('load data...')
    # load wrfoutput file only if not already loaded
    ds = salem.open_wrf_dataset(dspath)
    _, index = np.unique(ds['time'], return_index=True)
    ds = ds.isel(time=index)
    # only analyze after spinup time
    ds = ds.sel(time=times)

    # get nearest grid point coordinate to central grid location -> TAWES here
    cloc = loc_coords(ds.sel(time=ds.time[0]), central[0:2])

    print('Clipping on Tyrol region...')
    lat_start = central[0] - lat_view
    lat_end = central[0] + lat_view
    if (lat_start<=float(ds.lat.min()))|(lat_end>=float(ds.lat.max())):
        lat_start, lat_end = float(ds.lat.min()), float(ds.lat.max())
    lon_start = central[1] - lon_view
    lon_end = central[1] + lon_view
    if (lon_start<=float(ds.lon.min()))|(lon_end>=float(ds.lon.max())):
        lon_start, lon_end = float(ds.lon.min()), float(ds.lon.max())
    # select subset dataset
    ds_sub = ds.salem.subset(corners=((lon_start, lat_start), 
                                        (lon_end, lat_end)),
                             crs=salem.wgs84)
    if full_data:
        print('Adding WS at surface to data set...')
        WS2wrf(ds_sub)
        #'''
        print('Adding Surface Energy Balance to data set...')
        SFC_EBalance(ds_sub)
        print('Adding Temperature [°C] to data set...')
        ds_sub['TC'] = theta_to_temp(ds_sub.THETA, ds_sub.PRESSURE)
        ds_sub['dTdz'] = ds_sub.T2C - ds_sub.TC.sel(bottom_top=0)
        print('Adding relative humidity to data set...')
        ds_sub['RH'] = rh(ds_sub.QVAPOR, ds_sub.P + ds_sub.PB,
                          ds_sub.TC + 273.15)
        ds_sub['RH2'] = rh(ds_sub.Q2, ds_sub.PSFC,
                          ds_sub.T2C + 273.15)
        print('Adding dew point to data set...')
        ds_sub['TD'] = td(ds_sub.PRESSURE, ds_sub.QVAPOR)
        #'''

    sondes_list = [None, None, None]
    # select profile locations
    print('Add vertical profiles for selected profile locations...')
    for i, sonde in enumerate(sondes):
        # Skip location if not covered by dataset
        if ((sonde[1]<=ds.lon.min()) | (sonde[1]>=ds.lon.max()) |
            (sonde[0]<=ds.lat.min()) | (sonde[0]>=ds.lat.max())):
            print('Load profile for {}N {}E...'.format(sonde[0], sonde[1]))
            print('Skip; profile location not covered by dataset.')
        # use full dataset if location not in subset
        elif ((sonde[1]<=ds_sub.lon.min()) | (sonde[1]>=ds_sub.lon.max()) |
            (sonde[0]<=ds_sub.lat.min()) | (sonde[0]>=ds_sub.lat.max())):
            print('Load profile for {}N {}E...'.format(sonde[0], sonde[1]))
            # get nearest grid point coordinate to seefeld TAWES profile location
            sloc_x, sloc_y = loc_coords(ds.sel(time=ds.time[0]), sonde[0:2])
            dssonde = ds.sel(south_north=ds.south_north[sloc_y],
                                 west_east=ds.west_east[sloc_x])
            WS2wrf(dssonde)
            SFC_EBalance(dssonde)
            dssonde['TC'] = theta_to_temp(dssonde.THETA, dssonde.PRESSURE)
            dssonde['RH'] = rh(dssonde.QVAPOR,
                               dssonde.P + dssonde.PB, dssonde.TC + 273.15)
            dssonde['TD'] = td(dssonde.PRESSURE, dssonde.QVAPOR)
            sondes_list[i] = dssonde
        # use subset for other locations
        else:
            print('Load profile for {}N {}E...'.format(sonde[0], sonde[1]))
            # get nearest grid point coordinate to seefeld TAWES profile location
            sloc_x, sloc_y = loc_coords(ds_sub.sel(time=ds_sub.time[0]), sonde[0:2])
            dssonde = ds_sub.sel(south_north=ds_sub.south_north[sloc_y],
                                 west_east=ds_sub.west_east[sloc_x])
            WS2wrf(dssonde)
            SFC_EBalance(dssonde)
            dssonde['TC'] = theta_to_temp(dssonde.THETA, dssonde.PRESSURE)
            dssonde['RH'] = rh(dssonde.QVAPOR,
                               dssonde.P + dssonde.PB, dssonde.TC + 273.15)
            dssonde['TD'] = td(dssonde.PRESSURE, dssonde.QVAPOR)
            sondes_list[i] = dssonde
    return ds_sub, sondes_list, cloc


def elevation_load(dspath, sn_ind=[102, 106], we_ind=[117, 119]):
    '''
    load elevation from wrf output.
    
    Parameters
    ----------
    dspath : str
        path to wrf output.
    sn_ind : list/tuple
        bounding indices for south_north coordinate.
    we_ind : list/tuple
        bounding indices for west_east coordinate.

    Returns
    -------
    elevation : xarray dataarray
        dataarray with model terrain height.
    '''
    ds = salem.open_wrf_dataset(dspath)

    elevation = ds.HGT.sel(south_north=np.arange(ds.south_north[sn_ind[0]],
                                                 ds.south_north[sn_ind[1]]+1, 200),
                           west_east=np.arange(ds.west_east[we_ind[0]],
                                               ds.west_east[we_ind[1]]+1, 200),
                           time=ds.time[0])

    return elevation


def turb_load(path, times, **kwargs):
    '''
    load turbulence data.
    
    Parameters
    ----------
    path : str
        path to data.
    times : list/array
        list of times to be analyzed.

    Returns
    -------
    turb_data : pandas dataframe
        dataframe with desired turbulence data.
    '''
    turb_data = pd.read_csv(path, **kwargs)
    turb_data.index = pd.to_datetime(turb_data.index)
    turb_data = turb_data[(turb_data.index>=times[0]) & (turb_data.index<=times[-1])]
    turb_data = turb_data.where(turb_data['qcflag_all1_fc']>0)
    turb_data['le1'] = turb_data.le1.where(turb_data['qcflag_wq1']>0)
    turb_data['sh1'] = turb_data.sh1.where(turb_data['qcflag_wt1']>0)
    return turb_data


def load_rad(path, times, **kwargs):
    '''
    load radiation data.
    
    Parameters
    ----------
    path : str
        path to data.
    times : list/array
        list of times to be analyzed.

    Returns
    -------
    rad_data : pandas dataframe
        dataframe with desired radiation data.
    '''
    rad_data = pd.read_csv(path, **kwargs)
    rad_data = rad_data.set_index('TIMESTAMP')
    rad_data.index = pd.to_datetime(rad_data.index)
    rad_data = rad_data[(rad_data.index>=times[0]) & (rad_data.index<=times[-1])]
    return rad_data


def load_pseudoprofile(path, times):
    '''
    load pseudoprofile data.
    
    Parameters
    ----------
    path : str
        path to data.
    times : list/array
        list of times to be analyzed.

    Returns
    -------
    pp_data : pandas dataframe
        dataframe with desired pseudoprofile data
        (first column lowest elevation).
    heights : np.array
        Station heights for each station.
    '''
    data_dict = loadmat(path)
    datenums = data_dict['sdate'][:,0]
    timestamps = pd.to_datetime(datenums-719529, unit='D').round('1min')
    names = list()
    for station in data_dict['Stationsname'][0]:
        names.append(str(station[0]))
    heights = data_dict['Heights'][0]
    pp_data = pd.DataFrame(index=timestamps, data=data_dict['DATEN'], columns=names)
    pp_data = pp_data[(pp_data.index>=times[0]) & (pp_data.index<=times[-1])]

    return pp_data, np.squeeze(heights)


def load_measurements():
    '''
    Routine that loads measurement data.

    Returns:
    ----------
    station_data : pd.DataFrame
        Data from TAWES station
    files : list(pd.DataFrames)
        List with dataframes containing MOMAA data
    turb_data : list(pd.DataFrames)
        List with dataframes containing MOMAA turbulent flux data    
    rad_data : list(pd.DataFrames)
        List with dataframes containing MOMAA radiation data
    pp_data : pandas dataframe
        dataframe with desired pseudoprofile data
        (first column lowest elevation).
    heights : np.array
        Station heights for each station.
    '''
    # load TAWES station data
    print('load station data...')
    station_data = pd.read_csv(spath, delim_whitespace=True, header=0)
    station_data['datumsec'] = pd.to_datetime(station_data.datumsec/(24*60*60), unit='D').round('10min')
    station_data = station_data.set_index('datumsec')
    station_data = station_data[(station_data.index>=bound_times[0]) &
                                (station_data.index<=bound_times[1])]
    station_data['w'] = rh_to_mr(station_data.rf/100., station_data.tl,
                                 station_data.p, celsius=True, pascal=False)
    # load MOMAA station data
    files = list()
    for i, station in enumerate(mnames):
        station_file = pd.read_csv(datapath+station, sep=',', index_col=0, header=0)
        station_file.index = pd.to_datetime(station_file.index)
        station_file = station_file[(station_file.index>=bound_times[0]) &
                                    (station_file.index<=bound_times[1])]
        station_file['w'] = rh_to_mr(station_file.RHact_Avg/100., station_file.TAact_Avg,
                                     station_file.BPact_Avg, celsius=True, pascal=False)
        station_file['time'] = station_file.index
        files.append(station_file.resample('10min').mean())

    # load turbulent flux data
    print('load turbulence data...')
    turb_data_M04 = turb_load(turb_paths[0], times, header=0, index_col=0,
                              skiprows=[1])
    turb_data_M08 = turb_load(turb_paths[1], times, header=0, index_col=0,
                              skiprows=[1])
    files[2]['le1'] = (turb_data_M04.le1).resample('10min').median()
    files[-2]['le1'] = (turb_data_M08.le1).resample('10min').median()
    files[2]['sh1'] = (turb_data_M04.sh1).resample('10min').median()
    files[-2]['sh1'] = (turb_data_M08.sh1).resample('10min').median()


    # loaod pseudoprofile data
    print('load pseudoprofile data...')
    pp_data, heights = load_pseudoprofile(pseudopath, times)

    # load radiation data
    print('load radiation data...')
    rad_data_M04 = load_rad(datapath+rad_paths[0], times, header=0, 
                            skiprows=[0,2,3])
    rad_data_M08 = load_rad(datapath+rad_paths[1], times, header=0, 
                              skiprows=[0,2,3])
    files[2]['NRact_Avg'] = (rad_data_M04.LWin_Avg + rad_data_M04.SWin_Avg -
            rad_data_M04.LWout_Avg - rad_data_M04.SWout_Avg).resample('10min').mean()
    files[-2]['NRact_Avg'] = (rad_data_M08.LWin_Avg + rad_data_M08.SWin_Avg -
            rad_data_M08.LWout_Avg - rad_data_M08.SWout_Avg).resample('10min').mean()

    turb_data = [turb_data_M04, turb_data_M08]
    rad_data = [rad_data_M04, rad_data_M08]

    return (station_data, files, turb_data,
            rad_data, pp_data, heights)
