#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Dec 16 17:20:50 2022

@author: andreas

plot terrain contours, orthophoto and stations on it.
Needs environment earth-analytics-python to be activated.
To install that environment, run

conda env create earth_analytics_python.yml

in a terminal window or download the environment file
at https://github.com/earthlab/earth-analytics-python-env
and follow the instructions.
"""

import matplotlib.pyplot as plt
import geopandas as gpd
import rioxarray as rxr
import numpy as np
from rasterio.plot import plotting_extent
import earthpy.spatial as es
import earthpy.plot as ep
from shapely.geometry import Point
from matplotlib_scalebar.scalebar import ScaleBar


outpath = r'./'


# Station coordinates
TAWES = [47+19/60+29/3600, 11+10/60+32/3600, 1182]
gschwandtkopf = [47.3139, 11.1789, 1495]
brunschkopf = [47.3275, 11.1567, 1513]
hohemunde = [47.3500, 11.0833, 2592]
m02 = [47.32528418810252, 11.177498291879296, 1179]
m03 = [47.321334491368575, 11.17793144057643, 1260]
m04 = [47.321295824816595, 11.172384902233786, 1180]
m07 = [47.31910837997379, 11.16386563821771, 1200]
m08 = [47.32608207066923, 11.178603129172092, 1179]
m010 = [47.3223323125468, 11.173247921217945, 1186]
tower = [664404, 5243328]
vert_prof = [664518, 5243644]
hobolist = [[664735, 5243734],
         [664658, 5243730],
         [664473, 5243578],
         [664397, 5243498],
         [664330, 5243459],
         [664292, 5243371],
         [664221, 5243267],
         [664165, 5243210],
         [664106, 5243028],
         [664429, 5243541],
         [664257, 5243427],
         [664002, 5243154],
         [664543, 5243196],
         [664538, 5243205],
         [664530, 5243221],
         [664521, 5243239]]


# path to orthophoto
outp = r'../geofiles/orthophoto.tif'
# path to topography
out_topo = r'../geofiles/topo_hr.tif'

# colors, labels, markers, etc. for each station
colors = ['#F0E442', '#D55E00', '#0072B2',
        '#CC79A7', '#009E73', '#56B4E9',
        '0.9', '0.3', '0.7']
stationnames = ['M02', 'M03', 'M04', 'M07', 'M08', 'M10', 'TAWES', 'Tower', 'VP']
stations = [m02, m03, m04, m07, m08, m010, TAWES]
smarkers = ['P', 'X', 'P', 'X', 'P', 'X', 's', '^', '^']
markersizes = [250, 250, 250, 250, 250, 250, 160, 160, 160]

# colors, labels, markers, etc. for hobos
hlabels = ['HOBO 2 heights', None, None, None, None, None,
        None, None, None, None, None, None,
        None, None, None, 'HOBO 1 height']
hcolors = ['0.9', '0.9', '0.9', '0.9', '0.9', '0.9',
        '0.9', '0.9', '0.9', '0.5', '0.5', '0.5',
        '0.5', '0.5', '0.5', '0.5']
hlist = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', ' J', 'M', 'N', 'Q',
        ' ',' ',' ',' ']

plt.rc('font',**{'size': 12})

# properties fpor text boxes
props = dict(boxstyle='round', facecolor='white', alpha=0.5)

# values of major and minor terrain contours    
ter_range = np.arange(500, 3000, 20)
ter_minor = ter_range[np.where(ter_range%100!=0)]
ter_major = ter_range[np.where(ter_range%100==0)]

ortho_data = rxr.open_rasterio(outp, masked=True)
# Creating the plot extent object
ortho_plot_extent = plotting_extent(ortho_data[0], ortho_data.rio.transform())
# get reference system of orthofoto
ortho_crs = es.crs_check(outp)

# terrain data for contours
ds = rxr.open_rasterio(out_topo)
# reproject - topography, stations and orthophoto should all be in same projection
dss = ds.rio.reproject(ortho_crs)
# crop image to desired extent
dss = dss.where(((dss.y>=241900)&(dss.y<=244100.)&
                (dss.x<=66000.)&(dss.x>=62400.)), drop=True)

# get station coordinates in crs of raster image
stations_out = []
for station in stations:
    s = gpd.GeoDataFrame(geometry=[Point(station[1], station[0])],
                        crs='EPSG:4326').to_crs(ortho_crs)
    stations_out.append(s)
tow = gpd.GeoDataFrame(geometry=[Point(tower[0], tower[1])],
                    crs='EPSG:32632').to_crs(ortho_crs)
stations_out.append(tow)
vp = gpd.GeoDataFrame(geometry=[Point(vert_prof[0], vert_prof[1])],
                    crs='EPSG:32632').to_crs(ortho_crs)
stations_out.append(vp)
hobos_out = []
# same for HOBOs
for hobo in hobolist:
    h = gpd.GeoDataFrame(geometry=[Point(hobo[0], hobo[1])],
                        crs='EPSG:32632').to_crs(ortho_crs)
    hobos_out.append(h)


#%% plot itsellf
f, ax = plt.subplots(figsize=(10,10))
# terrain contours
ax.contour(dss.x, dss.y, dss.sel(band=1), ter_minor, colors='0.7',
        linewidths=0.5, zorder=1)
c_major = ax.contour(dss.x, dss.y, dss.sel(band=1), ter_major, colors='0.7',
                    linewidths=1, zorder=1)
plt.clabel(c_major, fontsize=10, inline=True, fmt = '%d m')
# orthophoto
ep.plot_rgb(ortho_data.values, rgb=[0, 1, 2], ax=ax,
            extent=ortho_plot_extent)
ax.add_artist(ScaleBar(1, loc='lower left', location='lower left'))
ax.set_ylim((242300., 243600.))
ax.set_xlim((62500., 64500.))
# labels
ax.text(0.42, 0.48, 'UB', transform=ax.transAxes,
        verticalalignment='top', bbox=props)
ax.text(0.50, 0.58, 'RI', transform=ax.transAxes,
        verticalalignment='top', bbox=props)
ax.text(0.74, 0.92, 'LB', transform=ax.transAxes,
        verticalalignment='top', bbox=props)
ax.text(0.67, 0.46, 'S1-4', transform=ax.transAxes, fontsize=10, color='0.9')
for i, station  in enumerate(stations_out):
    station.plot(ax=ax, label=stationnames[i], color=colors[i],
                markersize=markersizes[i], edgecolor='k', zorder=2, marker=smarkers[i])

for i, hobo  in enumerate(hobos_out):
    hobo.plot(ax=ax, label=hlabels[i], color=hcolors[i],
                markersize=160, edgecolor='k', zorder=2)
    ax.annotate(hlist[i], [hobo.geometry.x, hobo.geometry.y],
            fontsize=10, textcoords="offset points",
            verticalalignment='center', horizontalalignment='center',
            xytext=(-0.0, -1.0))

plt.legend(loc='upper left')

plt.savefig(outpath+'Fig2.pdf', bbox_inches="tight")
