#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov  7 14:03:23 2022

@author: Andreas Rauchöcker

These functions were used to create most plots of the Publication
"Cold-Air Pool Development in a small Alpine Valley" in QJRMS.

The variable doplot decides which Figure to plot:
'fig1' plots Figure 1, 'fig7' plots Figure 7, 'fig9' plots
Figure 9 and every other value plots Figures 4, 5, 6, 8 and 11.
Figure 2 requires a different python environment and 
therefore is included in a different script (ortho_plot.py).
Figure 3 and Figure 10 were not created within Python and are
therefore not reproduced with this script.
"""

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import salem
from wrf import rh

from plotting_routines import (m04, TAWES, point_data, do_fig9, plot_crossmap, station_plots_all,
                               domains_plot, station_plots_both, les_paperplot, pseudo_profile,
                               pseudo_scatter_mult, theta_to_temp, SFC_EBalance, outpath, times)
from dataload import load_measurements, load_data

#%% Selections
doplot = "fig"
datanames = ["../output_WRF/jan16/wrfout_40m_jan16",
             "../output_WRF/jan16_sms/wrfout_40m_jan16_sms"]
tendpaths = [r"../output_WRF/jan16/tend_40m_jan16.nc",
             r"../output_WRF/jan16_sms/tend_40m_jan16_sms.nc"]
meanpaths = [r"../output_WRF/jan16/windout_40m_jan16",
             r"../output_WRF/jan16_sms/windout_40m_jan16_sms"]

# select dataset to use: datanames[0] is the main simulation,
# datanames[1] the with the SMS-3DTKE parameterization
dataname = datanames[0]


#%% Font setting
plt.rc('font',**{'family':'serif','serif':['Helvetica']})
plt.rc('font',**{'size': 13})


#characters to replace in plotname
replace = ['-', ':', ' ', 'T', '.']
#time indices to plot in pseudoscatter plot
trange_scat = [60, 78, 96]

# colors for the individual pseudoprofiles
colors_pp = [[0., 0., 0.],
             [1., 0.3, 0.3],
             [0.9, 0.55, 0.55],
             [0.5, 0.5, 1.],
             [0.2, 0.2, 0.9]]
# time indices for pseudoprofile
pp_times2 = [130, 150, 168, 192, 216]
pp_times1 = [60, 84, 108, 130]


# station names for plots with 6 stations
stationlist_names = ['M02', 'M03', 'M04', 'M07', 'M08', 'M10']
stationlist_coords = [[47.32528418810252, 11.177498291879296],
                      [47.321334491368575, 11.17793144057643],
                      [47.321295824816595, 11.172384902233786],
                      [47.31910837997379, 11.16386563821771],
                      [47.32608207066923, 11.178603129172092],
                      [47.3223323125468, 11.173247921217945]]
# station names for plots with 3 stations
stationlist_names2 = [['M03', 'SLOPE'], ['M04', 'UB'], ['M08', 'LB']]
stationlist_coords2 = [[47.321334491368575, 11.17793144057643],
                      [47.321295824816595, 11.172384902233786],
                      [47.32608207066923, 11.178603129172092]]


def do_fig7(filenames, outpath="./"):
    """
    Function to plot both horizontal wind speed distribution of temperature
    and wind speed at the lowest model level and a crosssection of these parameters.

    Parameters
    ----------
    filenames : list(str)
        list containing paths to files that should be plotted.
    outpath : str
        directory to which the figures should be saved.
        Default is "./".
    """
    # name to add to plots
    add_name = ["", "_les"]
    plt.rc('font',**{'size': 24})
    for i, filename in enumerate(filenames):
        (dsm, [seefeld_prof, ibk_prof, munich_prof],
        cloc) = load_data(filename, TAWES, 0.1, 0.05, times, sondes=[])                
        plot_crossmap(filename, dsm, outpath, add_name[i])


if __name__=='__main__':
    if doplot.lower()=="fig1":
        # location for profile to plot height levels
        seefeld = [47.3300, 11.1878, 1180]
        # plot terrain for domains D1, D3, D4 and D5 in same figure
        terrain=list()
        geopath = r"../geofiles/"
        ds = salem.open_wrf_dataset(geopath+"geo_em.d01.nc")
        terrain.append((ds.HGT_M.where(ds.LANDMASK==1)).sel(time=ds.time[0]))
        ds = salem.open_wrf_dataset(geopath+"geo_em.d03.nc")
        terrain.append((ds.HGT_M.where(ds.LANDMASK==1)).sel(time=ds.time[0]))
        ds = salem.open_wrf_dataset(geopath+"geo_em.d04.nc")
        terrain.append((ds.HGT_M.where(ds.LANDMASK==1)).sel(time=ds.time[0]))
        ds_n = salem.open_wrf_dataset(geopath+"geo_em.d05.nc")
        terrain.append((ds_n.HGT_M.where(ds_n.LANDMASK==1)).sel(time=ds_n.time[0]))

        ds_sub = ds.salem.subset(corners=((ds_n.XLONG_M.min(), ds_n.XLAT_M.min()), 
                                            (ds_n.XLONG_M.max(), ds_n.XLAT_M.max())),
                                crs=salem.wgs84)
        terrain.append((ds_sub.HGT_M.where(ds.LANDMASK==1)).sel(time=ds.time[0]))
        (dsm, [seefeld_prof, ibk_prof, munich_prof],
        cloc) = load_data(datanames[0], TAWES, 0.1, 0.05, times, sondes=[seefeld])                
        domains_plot(terrain, seefeld_prof.isel(time=0))
    elif doplot.lower()=="fig7":
        do_fig7(datanames)
    elif doplot.lower()=="fig9":
        do_fig9(outpath, datanames, tendpaths, meanpaths)
    else:
        # do all plots for les run as well
        if "_les" in dataname:
            add_name = "_les"
        else:
            add_name = ""
        print('load WRF output from simulation...')
        ds = salem.open_wrf_dataset(dataname)
        _, index = np.unique(ds['time'], return_index=True)
        ds = ds.isel(time=index)
        print('calculate Temperature in °C...')
        ds['TC'] = theta_to_temp(ds.THETA, ds.PRESSURE)
        print('calculate relative humidity...')
        ds['RH'] = rh(ds.QVAPOR, ds.P + ds.PB, ds.TC + 273.15)
        print('calculate surface energy balance terms...')
        SFC_EBalance(ds)

        print('determine CAP periods...')
        # define when CAP forms
        timeline = point_data(ds, m04, corr=(-2, 0))
        cap = timeline.time.where(((timeline.TC.sel(bottom_top=3)-
                                    timeline.TC.sel(bottom_top=0))>=1.8))

        print('load measurmement data...')
        # load all measuremet data
        (station_data, files, [turb_data_M04, turb_data_M08],
        [rad_data_M04, rad_data_M08],
        pp_data, heights) = load_measurements()
        for file in files:
            file['time']=file.index
        ds = ds.sel(time=times)

        print('plot measurement data...')
        # plot measured station data
        station_plots_all(files, stationlist_coords, stationlist_names, cap,
                        outname='Fig4{}.pdf'.format(add_name), short=False, rel_hum=True,
                        stationdata=True)

        print('plot temperature values for cells within basin...')
        # plot scatter plot of all data points within the valley volume
        pseudo_scatter_mult(ds, 'TC', [33, 65], [56, 91], pp_data, heights,
                            trange=trange_scat, height=0, bench=3., add_name=add_name)

        print('plot pseudoprofiles for measurement data...')
        # plot pseudoprofiles at selected times
        pseudo_profile(pp_data, heights, pp_times2, '(b)',
                    colors_pp, add_name=add_name)
        pseudo_profile(pp_data, heights, pp_times1, '(a)',
                    colors_pp[1:], add_name=add_name)

        f = [files[1], files[2], files[4]]
        print('plot difference between simulation and measurement data...')
        station_plots_both(ds, f, stationlist_coords2, stationlist_names2, cap,
                        outname='Fig6{}.pdf'.format(add_name), short=False, rel_hum=True)

        if "_les" in dataname:
            # Fig11 only for LES run
            les_paperplot(ds, f, stationlist_coords2, stationlist_names2, cap,
                        outname='Fig11{}.pdf'.format(add_name))


