#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov  8 12:05:11 2021

@author: Andreas Rauchöcker

Functions needed for plotting WRF output, also against observations.
"""
import datetime
import numpy as np
import pandas as pd
import xarray as xr
import salem
import geopandas as gpd
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle, ConnectionPatch
from matplotlib.colors import LinearSegmentedColormap
from matplotlib import cm
import matplotlib.font_manager as fm
import matplotlib.dates as mdates
import matplotlib.units as munits
from netCDF4 import Dataset
from shapely.geometry import Point, LineString, Polygon
from haversine import haversine_vector, Unit
from pyproj import Transformer
from wrf import uvmet, getvar, to_np, vertcross, interpline, CoordPair


outpath = r'./'

# times to plot data
times = pd.date_range(start='1/16/2020 12:00:00', end='1/17/2020 12:00:00', freq='10min')
timeframe = pd.date_range(start='1/16/2020 00:00:00', end='1/17/2020 12:00:00', freq='10min')
#times = pd.date_range(start='1/16/2020 11:30:00', end='1/17/2020 12:00:00', freq='10min')

#%% select date format automatically 
formats = ['%y',     # ticks are mostly years
           '%b',     # ticks are mostly months
           '%d',     # ticks are mostly days
           '%H:%M',  # hrs
           '%H:%M',  # min
           '%S.%f', ]  # secs
# these can be the same, except offset by one level....
zero_formats = [''] + formats[:-1]
# ...except for ticks that are mostly hours, then its nice to have month-day
zero_formats[3] = '%d-%b'
offset_formats = ['',
                  '%Y',
                  '%b %Y',
                  '%d %b %Y',
                  '%d %b %Y',
                  '%d %b %Y %H:%M', ]

converter = mdates.ConciseDateConverter(formats=formats,
                                        zero_formats=zero_formats,
                                        offset_formats=offset_formats)
munits.registry[np.datetime64] = converter
munits.registry[datetime.date] = converter
munits.registry[datetime.datetime] = converter

#%% Font setting
plt.rc('font',**{'family':'serif','serif':['Helvetica']})
plt.rc('font',**{'size': 20})

font = {'fontname' : 'Liberation Serif'}

# properties fpor text boxes
props = dict(boxstyle='round', facecolor='white', alpha=0.5)

fontprops = fm.FontProperties(size=18)

#%% subplot indices
subplot_labels = ['(a)', '(b)', '(c)', '(d)', '(e)', '(f)', '(g)', '(h)']
#%% Coordinates
# coordinates of TAWES and important land marks
TAWES = [47+19/60+29/3600, 11+10/60+32/3600, 1182]
munich = [48.24, 11.552, 489]
innsbruck = [47.256165642, 11.339665308, 581]
seefeld = [47.3300, 11.1878, 1180]
leutasch = [47+22/60+15/3600, 11+8/60+33/3600, 1136]
scharnitz = [47+23/60+18/3600, 11+15/60+54/3600, 964]
telfs = [47.3071, 11.0682, 634]
mosern = [47.3113, 11.1493, 1206]
gschwandtkopf = [47.3139, 11.1789, 1495]
brunschkopf = [47.3275, 11.1567, 1513]
hohemunde = [47.3500, 11.0833, 2592]
reitherspitze = [47.3167, 11.2333, 2374]
rosshuette = [47.341278, 11.227897, 1745]
gehrenspitze = [47.386672, 11.135139, 2350]
m02 = [47.32528418810252, 11.177498291879296, 1179]
m03 = [47.321334491368575, 11.17793144057643, 1260]
m04 = [47.321295824816595, 11.172384902233786, 1180]
m07 = [47.31910837997379, 11.16386563821771, 1200]
m08 = [47.32608207066923, 11.178603129172092, 1179]
m010 = [47.3223323125468, 11.173247921217945, 1186]
tower = [664404, 5243328]
vert_prof = [664518, 5243644]
hobolist = [[664735, 5243734],
            [664658, 5243730],
            [664473, 5243578],
            [664397, 5243498],
            [664330, 5243459],
            [664292, 5243371],
            [664221, 5243267],
            [664165, 5243210],
            [664106, 5243028],
            [664429, 5243541],
            [664257, 5243427],
            [664002, 5243154],
            [664543, 5243196],
            [664538, 5243205],
            [664530, 5243221],
            [664521, 5243239]]


# names, coordinates, color and linestyle for plots with 6 stations
stationlist_names = ['M02', 'M03', 'M04', 'M07', 'M08', 'M10']
station_colors = [[0.2, 0.6, 0.2], [0.8, 0.3, 0.3], [0.45, 0.45, 0.8],
                  [0.8, 0.3, 0.3], [0.2, 0.6, 0.2], [0.45, 0.45, 0.8]]
station_linestyle = ['--', '-', '-', '--', '-', '--']
stationlist_coords = [m02[:-1], m03[:-1], m04[:-1], m07[:-1], m08[:-1], m010[:-1]]

# names, coordinates, color and linestyle for plots with 3 stations
stationlist_names2 = [['M03', 'SLOPE'], ['M04', 'UB'], ['M08', 'LB']]
station_colors2 = [[0.8, 0.3, 0.3], [0.45, 0.45, 0.8], [0.2, 0.6, 0.2]]
stationlist_coords2 = [m03[:-1], m04[:-1], m08[:-1]]

# markers for different station types: * for ventilated rotronic, P for unventilated rotronic, o for HOBO
markers = ['*', 'P', '*', 'P']
h_names = ['H', 'G', 'Tower_2m', 'Tower_HOBOsecondfloor',
           'Tower_HOBOthirdfloor', 'Ski_1217', 'Ski_1229',
           'Ski_1241', 'Ski_1246']
m_names = ['M04', 'M10', 'Tower_Rotronic', 'M03']

# landuse classes plus colors and labels representing them (urban, low vegetation, high vegetation)
lu_class = [[1, 0], [2, 7], [14, 15]]
colors = np.array([
'black',           #  Urban and Built-up Land
'cornflowerblue',     # Dryland Cropland and Pasture
'forestgreen',         # Forests
])
labels = ['Urban',
          'Grassland',
          'Forest']

#%% CONSTANTS
l_evap = 3.34e6           # latent heat of evaporation
l_melt = 2.23e5           # latent heat of fusion
gas_constant_air = 287    # specific gas constant for dry air [J kg-1 K-1]
gas_constant_water = 461  # specific gas constant for water [J kg-1 K-1]
heat_capacity = 1005      # specific heat for dry air [J kg-1 K-1]
norm_pressure = 1014      # sea level pressure [hPa]
stbolt = 5.67e-8          # Stefan Boltzmann Constant [W m-2 K-4]
g = 9.81                  # gravity [m s-2]

kappa = gas_constant_air / heat_capacity
epsilon = gas_constant_air / gas_constant_water

#%% base values
t0 = 273.15             # base Temperature[K]
e0 = 6.11               # base saturation vapor pressure [hPa]


#%% utility functions
def truncate_colormap(cmap, minval=0.0, maxval=1.0, n=100):
    '''
    use only parts of a predefined colormap 
    
    Parameters
    ----------
    cmap : colormap
        base colormap
    minval : float [0, 1]
        lower boundary of used colors (relative values).
        Default is 0.0 (lower bound of original color map).
    maxval : float [0, 1]
        upper boundary of used colors (relative values).
        Default is 1.0 (upper bound of original color map).
    n : int
        number of levels. Default is 100.
    '''
    new_cmap = LinearSegmentedColormap.from_list(
          'trunc({n},{a:.2f},{b:.2f})'.format(n=cmap.name, a=minval, b=maxval),
           cmap(np.linspace(minval, maxval, n)))
    return new_cmap 


def get_cone_factor(truelat1, truelat2):
    '''
    Calculate cone factor used for the (conic) map projection.
    
    Parameters
    ----------
    data : Xarray Dataset
        dataset containing

    Returns
    -------
    cone : float
        cone factor    
    '''
    if((np.fabs(truelat1 - truelat2) > 0.1) and
       (np.fabs(truelat2 - 90.) > 0.1)):
        cone = (np.log(np.cos(truelat1*np.pi/180.))
                - np.log(np.cos(truelat2*np.pi/180.)))
    
        cone = (cone /
                (np.log(np.tan((45.-np.fabs(truelat1/2.))*np.pi/180.))
                 - np.log(np.tan((45.-np.fabs(truelat2/2.))*np.pi/180.))))
    else:
        cone = np.sin(np.fabs(truelat1)*np.pi/180)
    return cone


def closest_point(coords, target):
    '''
    pick grid point that is closest to target coordinates

    Parameters
    ----------
    coords : zip iterable or list of tuples
        list of grid point coordinates
    target : list or tuple
        coordinates of target point

    Returns
    -------
    closest : tuple
        coordinates of closest target grid point
    '''

    coords = np.array(coords)
    # calculate haversine distance for each grid point relative to target
    new_dist = haversine_vector(coords, target, unit=Unit.METERS, comb=True)

    closest = coords[np.where(new_dist==new_dist.min())[1], :][0]
    return closest


def area_trans(ll_point, ul_point, lr_point, ur_point, grid_map):
    '''
    converts longitude latitude coordinates of line into Geodataframe object
    to be plotted over map object grid_map

    Parameters
    ----------
    ll_point : list or tuple
        longitude and latitude coordinates of lower left corner of area.
        Altitude might also be included, but no use for it so far
    grid_map : map object
        salem map object that contains the data

    Returns
    -------
    station_object : geopandas GeoDataFrame object
        geopandas GeoDataFrame object of the station
    '''
    grid_coords = grid_map.grid.ll_coordinates

    # get nearest grid point coordinate to start and end point of cross section
    area_coords = [closest_point(list(zip(grid_coords[1].flatten(),
                                          grid_coords[0].flatten())),
                                 ll_point),
                   closest_point(list(zip(grid_coords[1].flatten(),
                                          grid_coords[0].flatten())),
                                 ul_point),
                   closest_point(list(zip(grid_coords[1].flatten(),
                                          grid_coords[0].flatten())),
                                 ur_point),
                   closest_point(list(zip(grid_coords[1].flatten(),
                                          grid_coords[0].flatten())),
                                 lr_point)]
    # get nearest grid point index to start and end point of cross section
    [[(ll_y, ll_x),
      (ul_y, ul_x),
      (lr_y, lr_x),
      (ur_y, ur_x)]] = [np.where(grid_coords[0] == area_coords[0][1]),
                        np.where(grid_coords[0] == area_coords[1][1]),
                        np.where(grid_coords[0] == area_coords[2][1]),
                        np.where(grid_coords[0] == area_coords[3][1])]

    # convert to geopandas object that can be plotted
    area_object = gpd.GeoDataFrame(geometry=[Polygon([(ll_x, ll_y),
                                                      (ul_x, ul_y),
                                                      (lr_x, lr_y),
                                                      (ur_x, ur_y)])])

    return area_object


def cross_trans(start_point, end_point, grid_map):
    '''
    converts longitude latitude coordinates of line into Geodataframe object
    to be plotted over map object grid_map

    Parameters
    ----------
    start_point : list or tuple
        longitude and latitude coordinates of start point to be plotted.
        Altitude might also be included, but no use for it so far
    end_point : list or tuple
        longitude and latitude coordinates of end point to be plotted.
        Altitude might also be included, but no use for it so far
    grid_map : map object
        salem map object that contains the data

    Returns
    -------
    station_object : geopandas GeoDataFrame object
        geopandas GeoDataFrame object of the station
    '''
    grid_coords = grid_map.grid.ll_coordinates

    # get nearest grid point coordinate to start and end point of cross section
    cross_coords = [closest_point(list(zip(grid_coords[1].flatten(),
                                           grid_coords[0].flatten())),
                                  start_point),
                    closest_point(list(zip(grid_coords[1].flatten(),
                                           grid_coords[0].flatten())),
                                  end_point)]
    # get nearest grid point index to start and end point of cross section
    [(start_y, start_x),
     (end_y, end_x)] = [np.where(grid_coords[0] == cross_coords[0][1]),
                        np.where(grid_coords[0] == cross_coords[1][1])]

    # convert to geopandas object that can be plotted
    cross_object = gpd.GeoDataFrame(geometry=[LineString([(start_x, start_y),
                                                          (end_x, end_y)])])

    return cross_object


def station_trans(station, grid_map):
    '''
    converts longitude latitude coordinates into Geodataframe object to be
    plotted over map object grid_map

    Parameters
    ----------
    station : list or tuple
        longitude and latitude coordinates of stations to be plotted.
        Altitude might also be included, but no use for it so far
    grid_map : map object
        salem map object that contains the data

    Returns
    -------
    station_object : geopandas GeoDataFrame object
        geopandas GeoDataFrame object of the station
    station_coords : tuple/list
        x and y coordinate of station
    '''
    grid_coords = grid_map.grid.ll_coordinates
    # get nearest grid point coordinate to station
    station_coord = closest_point(list(zip(grid_coords[1].flatten(),
                                            grid_coords[0].flatten())),
                                  station)

    # get nearest grid point index to station
    station_y, station_x = np.where(grid_coords[0] == station_coord[1])

    # convert to coordinate pair that can be plotted
    station_object = gpd.GeoDataFrame(geometry=[Point(station_x, station_y)])

    return station_object, [station_x, station_y]

def loc_coords_raw(file, station, wrfinner=True):
    '''
    locates closest grid point to point in longitude and lattitude coordinates
    
    Parameters
    ----------
    file : salem dataset
        salem dataset that contains the data
    station : tuple, list
        (lon, lat) coordinates to be located
    wrfinner : bool
        Whether input file is wrfout file or wrfinput. Default is wrfoutput.
    
    Returns
    ----------
    loc_x : int
        x-coordinate in salem map
    loc_y : int
        y-coordinate in salem map
    '''
    if wrfinner:
        lat = 'XLAT'
        lon = 'XLONG'
    else:
        lat = 'XLAT_M'
        lon = 'XLONG_M'
    loc_coord = closest_point(list(zip(file[lat].data.flatten(),
                                       file[lon].data.flatten())),
                              station[0:2])
    loc_y = np.where(file[lat] == loc_coord[0])[0][0]
    loc_x = np.where(file[lon] == loc_coord[1])[1][0]
    return loc_x, loc_y

def loc_coords(file, station):
    '''
    locates closest grid point to point in longitude and lattitude coordinates
    
    Parameters
    ----------
    file : salem dataset
        salem dataset that contains the data
    station : tuple, list
        (lon, lat) coordinates to be located
    
    Returns
    ----------
    loc_x : int
        x-coordinate in salem map
    loc_y : int
        y-coordinate in salem map
    '''

    loc_coord = closest_point(list(zip(file.lat.data.flatten(),
                                       file.lon.data.flatten())),
                              station[0:2])
    loc_x = np.where(file.lon == loc_coord[1])[1][0]
    loc_y = np.where(file.lat == loc_coord[0])[0][0]
    return loc_x, loc_y


def pointcoord_transform(point, EPSG_in=32632, EPSG_out=4326):
    '''
    transforms coordinates to different spatial reference frame.

    Parameters
    ----------
    point : list or tuple
        x and y coordinates in input coordinates (x coordinate 1st, y 2nd).
    EPSG_in : int
        EPSG code of input reference frame.
    EPSG_out : int
        EPSG code of output reference frame.

    Returns
    ----------
    outpoint : list
        x and y coordinates in output reference frame (x coordinate 1st, y 2nd).
    '''

    proj = Transformer.from_crs(EPSG_in, EPSG_out, always_xy=True)
    x2, y2 = proj.transform(point[1],point[0])
    return list((y2, x2))


def point_data(ds_in, station, corr=(None, None),
               stag_x=False, stag_y=False):
    '''
    Select data at desired point location only.

    Parameters
    ----------
    ds_in : salem/xr DataSet
        DataSet containing data.
    station : list or tuple
        y and x coordinate of desired location.
    corr : tuple
        . Default is (None, None).
    stag_x : bool
        True if data is staggered in x-direction (default is False).
    stag_y : bool
        True if data is staggered in y-direction (default is False).

    Returns
    ----------
    file : salem/xr DataSet
        DataSet only at location desired
    '''
    # get nearest grid point coordinate to timeline location
    loc_x, loc_y = loc_coords(ds_in, station)
    if corr[0]:
        loc_x = loc_x + corr[0]
    elif corr[1]:
        loc_y = loc_y + corr[1]
    if stag_y:
        file = (ds_in.sel(south_north=ds_in.south_north[loc_y],
                          west_east=ds_in.west_east[loc_x]) +
                ds_in.sel(south_north=ds_in.south_north[loc_y+1],
                          west_east=ds_in.west_east[loc_x]))/2
    elif stag_x:
        file = (ds_in.sel(south_north=ds_in.south_north[loc_y],
                          west_east=ds_in.west_east[loc_x]) +
                ds_in.sel(south_north=ds_in.south_north[loc_y],
                          west_east=ds_in.west_east[loc_x+1]))/2
    else:
        # select timeline location
        file = ds_in.sel(south_north=ds_in.south_north[loc_y],
                               west_east=ds_in.west_east[loc_x])
    return file

#%% Add data to WRF output
def SFC_EBalance(ds, noah_mp=True):
    '''
    add surface energy balance components to WRF output dataset
    
    Parameters
    ----------
    ds : Xarray Dataset without all Surface energy balance components

    Returns
    -------
    ds : Xarray Dataset with Surface energy balance components
    '''
    ds['SW_net'] = (1. - ds.ALBEDO) * (ds.SWNORM)
    #ds['SW_net'] = ds.SWDNB - ds.SWUPB
    ds['SW_net'].attrs['units'] = 'W m-2'
    ds['SW_net'].attrs['description'] = 'net Shortwave Radiation at ground'
    ds['SW_out'] = (0. - ds.ALBEDO) * (ds.SWNORM)
    ds['SW_out'].attrs['units'] = 'W m-2'
    ds['SW_out'].attrs['description'] = 'Outgoing Shortwave Radiation at ground'
    ds['LW_out'] = -1. * stbolt * ds.TSK**4
    ds['LW_out'].attrs['units'] = 'W m-2'
    ds['LW_out'].attrs['description'] = 'outgoing Longwave Radiation at ground'
    ds['LW_net'] = ds.EMISS * (ds.GLW + ds.LW_out)
    ds['LW_net'].attrs['units'] = 'W m-2'
    ds['LW_net'].attrs['description'] = 'net Longwave Radiation at ground'

    if noah_mp:
        ds['GRDFLX'] = -1. * ds.GRDFLX
        ds['RNET'] = ds.FSA - ds.FIRA
    else:
        ds['RNET'] = ds.SW_net + ds.LW_net
    ds['RNET'].attrs['units'] = 'W m-2'
    ds['RNET'].attrs['description'] = 'net Radiation at ground'
    ds['SFC_EB'] = ds.RNET + ds.GRDFLX - ds.HFX - ds.LH
    ds['SFC_EB'].attrs['units'] = 'W m-2'
    ds['SFC_EB'].attrs['description'] = 'Surface Energy Balance'
    if hasattr(ds,'pyproj_srs'):
        ds['SW_net'].attrs['pyproj_srs'] = ds.pyproj_srs
        ds['LW_out'].attrs['pyproj_srs'] = ds.pyproj_srs
        ds['LW_net'].attrs['pyproj_srs'] = ds.pyproj_srs
        ds['RNET'].attrs['pyproj_srs'] = ds.pyproj_srs
        ds['SFC_EB'].attrs['pyproj_srs'] = ds.pyproj_srs
    return ds


def WS2wrf(ds):
    '''
    add 10m wind speed to WRF output dataset
    
    Parameters
    ----------
    ds : Xarray Dataset without 10m wind speed

    Returns
    -------
    ds : Xarray Dataset with 10m wind speed
    '''
 
    ds['WS10'] = np.sqrt(ds.U10**2+ds.V10**2)
    ds['WS10'].attrs['units'] = 'm s-1'
    ds['WS10'].attrs['description'] = 'WIND SPEED AT 10 M'
    if hasattr(ds,'pyproj_srs'):
        ds['WS10'].attrs['pyproj_srs'] = ds.pyproj_srs

    return ds


def theta_to_temp(pot_temp, pressure, celsius=True, pascal=False):
    '''
    calculates temperature from potential temeperature and pressure.

    Parameters
    ----------
    pot_temp : float or array-type
        Potential temperature
    pressure : float or array-type
        Pressure at same level as potential temperature. 
        If array-type, needs to be of same dimension as pot_temp.
    celsius : bool
        Gives result in celsius if input in Kelvin if True (default is True).
    pascal : bool
        Converts pressure from hPa to Pa if true (default is True).

    Returns
    -------
    temp : float or array-type
        Temperature
    '''
    if pascal:
        pressure = pressure * 100
    temp = pot_temp * (pressure / norm_pressure) ** kappa
    if celsius:
        temp = temp - 273.15

    return temp


def ms_to_knots(wind):
    '''converts wind speed from meters per second to knots.'''
    return wind * 1.9438445


def sat_pressure(temp, celsius=True):
    '''
    Calculate saturation water vapor pressure from temperature
    
    Parameters
    ----------
    temp : float or array-type
        Temperature
    celsius : bool
        Converts temperature from celsius to Kelvin if True (default is True).

    Returns
    -------
    e_sat : float or array-type
        Saturation vapor pressure.
    '''
    if celsius:
        temp = temp + 273.15
    e_sat = e0 * np.exp(l_evap/gas_constant_water * (1/t0 - 1/temp))
    return e_sat


def rh_to_mr(rh, temp, pressure, pascal=False, celsius=True):
    '''
    Calculate mixing ratio rfrom relative humidity from temperature and pressure.
    
    Parameters
    ----------
    rh : float or array-type
        Relative humidity.
    temp : float or array-type
        Temperature
    pressure : float or array-type
        Pressure at same level as temperature. 
        If array-type, needs to be of same dimension as temp.
    pascal : bool
        Converts pressure from hPa to Pa if true (default is True).
    celsius : bool
        Converts temperature from celsius to Kelvin if True (default is True).

    Returns
    -------
    w : float or array-type
        mixing ratio.
    '''
    if pascal:
        pressure = pressure * 100.
    e_sat = sat_pressure(temp, celsius=celsius)
    w_sat = epsilon * e_sat / (pressure - e_sat)
    w = rh * w_sat
    return w


def coord_trans(angle_rad, u, v):
    '''
    rotates wind components into new coordinates system
    Parameters
    ----------
    angle_rad : np.Array
        angle of rotation in rad
    u : np.Array
        wind in east
    v : np.Array
        wind in north
    Returns
    -------
    u_rot : np.Array
        rotated u-component of wind
    v_rot : np.Array
        rotated v-component of wind
    '''
    u_rot = u*np.cos(angle_rad) + v*np.sin(angle_rad)
    v_rot = v*np.cos(angle_rad) - u*np.sin(angle_rad)
    # set very small floats to 0
    u_rot[np.where(np.abs(u_rot)<1e-10)] = 0.
    v_rot[np.where(np.abs(v_rot)<1e-10)] = 0. 

    return u_rot, v_rot


#%% plot wind speeds as vector
def windarr2map(ax, smap, ds, height = 0, surface=False, diff=False,
                start=1, step=2, xx=None, yy=None, labels=dict(),
                **kwargs):
    '''
    Purpose
    -------
    add windarrows to a map

    author: Maria Siller, adapted by Andreas Rauchöcker

    Parameters
    ----------
    ax : axis handle
    smap : salem map object
        salem map created by "map = salem.Map(ds.salem.grid)"
    ds : salem dataset object
        wrf dataset eg. generated by salem
    height : int
        index of desired height level
    surface : bool
        if True, plot surface wind (using ds['U10'] and ds['V10'])
        default is False (using ds['U'] and ds['V'])
    diff : bool
        if True, plot wind difference (using ds['U10'] and ds['V10'])
    start : int
        index of first barb in x/y direction. default is 1.
    step : int
        barb every step-th gridpoint. default is 2.
    width : float
        width of one vector unit. default is 0.005.
    xx/yy: matplotlib transforms (will be computed if missing)
    labels: dict of labels - NOT USED BY NOW
            -> may be generated by map,labels = set_map(map,'par')

    Returns
    -------
    ax: axis handle
    '''

    cone_fact = get_cone_factor(ds.TRUELAT1, ds.TRUELAT2)
    if surface:
        labels['wind'] = 'wind@10m'
        if diff:
            u, v = uvmet(ds.u_diff_10m, ds.v_diff_10m, ds.lat, ds.lon,
                         ds.STAND_LON, cone_fact)
        else:
            u, v = uvmet(ds.U10, ds.V10, ds.lat, ds.lon, ds.STAND_LON,
                     cone_fact)
    else:
        labels['wind'] = 'wind'
        u, v = uvmet(ds.U.sel(bottom_top=height),
                  ds.V.sel(bottom_top=height), 
                  ds.lat, ds.lon, ds.STAND_LON, cone_fact)

    # transform coordinates to the map reference system and plot the arrows
    if xx is None or yy is None: 
        xx, yy = smap.grid.transform(ds.west_east.values, ds.south_north.values,
                                    crs=ds.salem.grid.proj)

    # Quiver only every step-th grid point
    uu = ms_to_knots(u)[start::step, start::step]
    vv = ms_to_knots(v)[start::step, start::step]
    xx = xx[start::step]
    yy = yy[start::step]    
    
    # windarr
    ax.barbs(xx, yy, ms_to_knots(uu), ms_to_knots(vv), length=6)
      
    return ax


#%% plot map of desired parameter from desired dataset
def level_plot(ax, file, param, height, station_coords=None,
               title='', cross=[], ctitle='', cmap='OrRd', cext=(0,10),
               sub=False, sub40=False, wind_quiver=False, legend=False, 
               sub_label=None, cbar=True, cextend='both', **kwargs):
    '''
    Plot selected parameter from WRFoutfput file selected domain with coarse
    representation of underlying terrain and country boundaries.

    Parameters
    ----------
    ax : axes handle
        handle for axes to be plotted on
    file : salem dataset
        salem dataset that contains the data
    param : str
        name of parameter 
    height : int
        height level to plot
    station_coords : list or tuple
        longitude and latitude coordinates of stations to be plotted.
        Altitude might also be included, but no use for it so far.
        Default is (None, None).
    title : str
        Str to be displayed as title
    cross : tuple
        tuple of cordinates used for plotting cross section
    ctitle : str
        Parameter name and unit to be desplayed in title of colormap
    cmap : str
        name of colormap to be used for plotting. Default is 'OrRd'
    cext : tuple
        tuple defining upper and lower limit for colormap
    sub : bool
        Defines whether plotted as subplot. Default is False.
    sub40 : bool
        Defines whether plotted as subplot with 40m resolution.
        Default is False.
    wind_quiver : bool
        option to overlay data with quiver plot, Default is False.
    legend : bool
        option to plot legend, Default is False.
    sub_label : str
        label for subplot if it is one. default is None.
    cbar : bool
        Whether or not to plot colorbar. Default is True.
    cextend : str
        defines which colorbar extensions are activated. Default is 'both'.

    Returns
    -------
    ax : axes handle
    '''
    plt.rc('font',**{'size': 24})

    smap = salem.Map(file.salem.grid)
    smap = file.HGT.salem.get_map(countries=True)
    smap.set_data(file[param].sel(bottom_top=height)) # the variable that you want to plot
    smap.set_cmap(cmap) #define the colors of the 2d map
    smap.set_levels(np.arange(cext[0], cext[1]+1, 1.))
    smap.set_extend(cextend)
    smap.set_scale_bar(location=(0.2, 0.1), length=None, maxlen=0.20,
                       text_kwargs = {'weight': 'semibold', 'fontsize': 22},
                       linewidth=5)
    smap.visualize(addcbar=cbar, title='{}'.format(title),
                   cbar_title= '{} '.format(ctitle), ax=ax, **kwargs)
    if station_coords:
        # transform and plot station coordinates
        station, sxy = station_trans(station_coords[0:2], smap)
        ax = station.plot(ax=ax, marker='X', markersize=300, color='k',
                          label='M0MAA 04')
    if cross:
        clabel = 'Cross section'
        for cs in cross:
            # transform and plot cross section coordinates
            crosssection = cross_trans(cs[0], cs[1], smap)
            ax = crosssection.plot(ax=ax, marker='o', markersize=300, color='k',
                                   linestyle='-', linewidth=3,
                                   label=clabel)
        a, axy = station_trans(cs[0][0:2], smap)
        b, bxy = station_trans(cs[-1][0:2], smap)
        ax.text(axy[0]-10, axy[1], 'A', verticalalignment='bottom', weight='bold')
        ax.text(bxy[0], bxy[1], 'B', verticalalignment='top', weight='bold')
    xx, yy = smap.grid.transform(file.west_east.values,
                                 file.south_north.values,
                                 crs=file.salem.grid.proj)

    ter_range = np.arange(500, 3000, 100)
    ter_minor = ter_range[np.where(ter_range%1000!=0)]
    ter_major = ter_range[np.where(ter_range%1000==0)]
    try:
        ax.contour(xx, yy, file.HGT, ter_minor, colors='dimgrey')
        contour_major = ax.contour(xx, yy, file.HGT, ter_major, colors='dimgrey')
    except AttributeError:
        ax.contour(xx, yy, file.HGT_M, ter_minor, colors='dimgrey')
        contour_major = ax.contour(xx, yy, file.HGT_M, ter_major, colors='dimgrey')
    plt.clabel(contour_major, fontsize=10, inline=True, fmt = '%d m')
    if wind_quiver:
        # add quiver plot for wind on top
        if sub:
            ax = windarr2map(ax, smap, file, surface=True, start=2, step=4)
        elif sub40:
            ax = windarr2map(ax, smap, file, surface=True, start=4, step=8)
        else:
            ax = windarr2map(ax, smap, file, surface=True, start=2, step=2)

    if sub_label:
        ax.text(0.03, 0.95, sub_label, verticalalignment='top',
                transform=ax.transAxes, bbox=dict(facecolor='white', alpha=0.5))
    if legend:
        ax.legend()

    return ax


def terrain_plot(ax, file, station_coords=(False,False), cross=[],
                 title=None, cmap='terrain', cext=(-700,3500),
                 ter_contours=False, domain=0, close=False, cbar=False,
                 cextend='both', **kwargs):
    '''
    Plot selected parameter from WRFoutfput file selected domain with coarse
    representation of underlying terrain and country boundaries.

    Parameters
    ----------
    ax : axes handle
        handle for axes to be plotted on
    file : salem dataset
        salem dataset that contains the data
    station_coords : list or tuple
        longitude and latitude coordinates of stations to be plotted.
        Altitude might also be included, but no use for it so far
    cross : tuple
        tuple of cordinates used for plotting cross section
    title : str
        Parameter name and unit to be desplayed in title and on colormap
    cmap : str
        name of colormap to be used for plotting. Default is 'OrRd'
    cext : tuple
        tuple defining upper and lower limit for colormap
    ter_contours : bool
        option to overlay data with contour lines of terrain height.
        Default is False.
    domain : int
        WRF domain number. Default is 0 (no specified WRF domain)
    close : bool
        If True, plots more landmarks than only centered ploint and
        domain boundaries. Default is False.
    cbar : bool
        If True, plot cbar. Default is False.
    cextend : str
        defines which colorbar extensions are activated. Default is 'both'.

    Returns
    -------
    ax : axes handle
    '''
    fsize = 22        # font size for text
    tw = 550   # text weight
    if domain==1:
        xint, yint = 10, 10
    elif domain==2:
        xint, yint = 3, 3
    elif domain==3:
        xint, yint = 1, 1
    elif domain==4:
        xint, yint = 0.1, 0.1
    else:
        xint, yint = 0.02, 0.01
    if close:
        xint, yint = 0.02, 0.01
    smap = salem.Map(file.salem.grid)
    smap.set_data(file) # the variable that you want to plot
    smap.set_extend(cextend)
    smap.set_cmap(cmap) # define the colors of the 2d map
    smap.set_vmin(cext[0])
    smap.set_vmax(cext[1])
    smap.set_scale_bar(location=(0.85, 0.9), length=None, maxlen=0.25,
                       text_kwargs = {'weight': 'semibold', 'fontsize': 22},
                       linewidth=5)
    smap.set_lonlat_contours(xinterval=xint, yinterval=yint)
    topo = smap.visualize(addcbar=cbar, title=title,
                          cbar_title= 'Terrain height [m a.g.l.]', ax=ax,
                          **kwargs)


    if ter_contours:
        xx, yy = smap.grid.transform(file.west_east.values,
                                     file.south_north.values,
                                     crs=file.salem.grid.proj)
        # add contour lines every 100m
        if close:
            ter_range = np.arange(100, 1600, 20)
            ter_minor = ter_range[np.where(ter_range%100!=0)]
            ter_major = ter_range[np.where(ter_range%100==0)]
        else:
            ter_range = np.arange(100, 4000, 200)
            ter_minor = ter_range[np.where(ter_range%1000!=0)]
            ter_major = ter_range[np.where(ter_range%1000==0)]
        ax.contour(xx, yy, file, ter_minor, colors='dimgrey')
        contour_major = ax.contour(xx, yy, file, ter_major, colors='dimgrey')
        plt.clabel(contour_major, fontsize=10, inline=True, fmt = '%d m')

    # add inner domain boundaries
    if domain==1:
        # domain 2:
        ax.add_patch(Rectangle((184.524, 116.6667), 375./3., 285.7/3., alpha=1, linewidth=3, fill=False))
        ax.text(185, 98, 'D2', fontsize=fsize, weight=tw)
        # domain 3:
        ax.add_patch(Rectangle((226.1905, 151.1905), 500./9., 357./9., alpha=1, linewidth=3, fill=False))
        ax.text(226, 132, 'D3', fontsize=fsize, weight=tw)
        ax.text(0.5, 0.95, '$\Delta x = 9$ km', transform=ax.transAxes,
                verticalalignment='top', horizontalalignment='center', 
                bbox=props, fontsize=fsize)
    elif domain==2:
        ax.add_patch(Rectangle((169.8413, 136.508), 666.6667/3., 496.8254/3.,alpha=1, linewidth=3, fill=False))
        ax.add_patch(Rectangle((249.7354, 237.037), 380.95/15, 333.33/15,alpha=1, linewidth=3, fill=False))
        ax.text(170, 130, 'D3', fontsize=fsize, weight=tw)
        ax.text(250, 255, 'D4', fontsize=fsize, weight=tw)
        ax.text(0.5, 0.95, '$\Delta x = 3$ km', transform=ax.transAxes,
                verticalalignment='top',  horizontalalignment='center', 
                bbox=props, fontsize=fsize)
    elif domain==3:
        ax.add_patch(Rectangle((178.5714, 225.), 285.7/5., 250./5.,alpha=1, linewidth=3, fill=False))
        ax.text(179, 280, 'D4', fontsize=fsize, weight=tw)
        ax.text(0.45, 0.95, '$\Delta x = 1$ km', transform=ax.transAxes,
                verticalalignment='top',  horizontalalignment='center', 
                bbox=props, fontsize=fsize)
    elif domain==4:
        ax.text(0.5, 0.95, '$\Delta x = 200$ m', transform=ax.transAxes,
                verticalalignment='top',  horizontalalignment='center', 
                bbox=props, fontsize=fsize)
        if close:
            ms, msxy = station_trans(mosern[0:2], smap)
            ms.plot(ax=ax, marker='.', markersize=300, color='k')
            ax.text(msxy[0]-30, msxy[1]-27, 'Mösern', fontsize=fsize, weight=tw)
            gk, gkxy = station_trans(gschwandtkopf[0:2], smap)
            ax.text(gkxy[0]-60, gkxy[1]-20, 'Gschwandtkopf', fontsize=fsize, weight=tw)
            bk, bkxy = station_trans(brunschkopf[0:2], smap)
            ax.text(bkxy[0]-70, bkxy[1]+10, 'Brunschkopf', fontsize=fsize, weight=tw)
            m3, m3xy = station_trans(m03[0:2], smap)
            m3.plot(ax=ax, marker='.', markersize=50, color='k')
            ax.text(m3xy[0]-0, m3xy[1]-22, 'M03', fontsize=fsize, weight=tw)
            m4, m4xy = station_trans(m04[0:2], smap)
            m4.plot(ax=ax, marker='o', markersize=75, color='k',
                              label='MOMAA+EB')
            ax.text(m4xy[0]-25, m4xy[1]-25, 'M04', color='k', fontsize=fsize, weight=tw)
            m7, m7xy = station_trans(m07[0:2], smap)
            m7.plot(ax=ax, marker='.', markersize=50, color='k')
            ax.text(m7xy[0]-35, m7xy[1]+10, 'M07', fontsize=fsize, weight=tw)
            m8, m8xy = station_trans(m08[0:2], smap)
            m8.plot(ax=ax, marker='o', markersize=75, color='k')
            ax.text(m8xy[0]-5, m8xy[1]+5, 'M08', color='k', fontsize=fsize, weight=tw)
            station, sxy = station_trans(TAWES[0:2], smap)
            station.plot(ax=ax, marker='X', markersize=300, color='k',
                              label='TAWES')
            ax.text(sxy[0]-95, sxy[1]+5, 'TAWES', fontsize=fsize, weight=tw)
        else:
            ax.add_patch(Rectangle((217.84, 199.02), 54.35, 50.21,alpha=1, 
                                   linestyle="-", linewidth=3, fill=False))
            lt, ltxy = station_trans(leutasch[0:2], smap)
            lt.plot(ax=ax, marker='o', markersize=100, color='k')
            ax.text(ltxy[0]-5, ltxy[1]+5, 'Leutasch', fontsize=fsize, weight=tw,
                    horizontalalignment='right', verticalalignment='bottom')
            sn, snxy = station_trans(scharnitz[0:2], smap)
            sn.plot(ax=ax, marker='o', markersize=100, color='k')
            ax.text(snxy[0]+5, snxy[1]+5, 'Scharnitz', fontsize=fsize, weight=tw,
                    horizontalalignment='left', verticalalignment='bottom')
            tf, tfxy = station_trans(telfs[0:2], smap)
            tf.plot(ax=ax, marker='o', markersize=300, color='k',
                              label='Telfs')
            ax.text(tfxy[0]-10, tfxy[1]-10, 'Telfs', fontsize=fsize, weight=tw,
                    horizontalalignment='right', verticalalignment='top')
    else:
        if close:
            ax.text(0.50, 0.95, '$\Delta x = 40$ m', transform=ax.transAxes,
                    verticalalignment='top',  horizontalalignment='center', 
                    bbox=props, fontsize=fsize)
            ax.add_patch(Rectangle((215.386, 126.92), 134.61, 123.08, alpha=1, 
                                   linestyle="--", linewidth=2, fill=False))
            # transform and plot cross stations
            sf, sfxy = station_trans(seefeld[0:2], smap)
            sf.plot(ax=ax, marker='o', markersize=200, color='k')
            ax.text(sfxy[0]-0, sfxy[1]+8, 'Seefeld', fontsize=fsize, weight=tw,
                    horizontalalignment='center', verticalalignment='bottom')
            ms, msxy = station_trans(mosern[0:2], smap)
            ms.plot(ax=ax, marker='o', markersize=200, color='k')
            ax.text(msxy[0]-0, msxy[1]+10, 'Mösern', fontsize=fsize, weight=tw,
                    horizontalalignment='center', verticalalignment='bottom')
            gk, gkxy = station_trans(gschwandtkopf[0:2], smap)
            ax.text(gkxy[0]-65, gkxy[1]-20, 'Gschwandtkopf', fontsize=fsize, weight=tw)
            bk, bkxy = station_trans(brunschkopf[0:2], smap)
            ax.text(bkxy[0]-65, bkxy[1]+0, 'Brunschkopf', fontsize=fsize, weight=tw)
            m2, m2xy = station_trans(m02[0:2], smap)
            m2.plot(ax=ax, marker='o', markersize=60, color='k',
                              label='MOMAA')
            ax.text(m2xy[0]+5, m2xy[1]-5, 'M02', fontsize=fsize, weight=tw,
                    horizontalalignment='left', verticalalignment='top')
            m3, m3xy = station_trans(m03[0:2], smap)
            m3.plot(ax=ax, marker='o', markersize=60, color='k')
            ax.text(m3xy[0]+5, m3xy[1]-10, 'M03', fontsize=fsize, weight=tw,
                    horizontalalignment='left', verticalalignment='top')
            m4, m4xy = station_trans(m04[0:2], smap)
            m4.plot(ax=ax, marker='o', markersize=120, color='k',
                              label='MOMAA+EB')
            ax.text(m4xy[0]-0, m4xy[1]-10, 'M04', color='k', fontsize=fsize, weight=tw,
                    horizontalalignment='center', verticalalignment='top')
            m7, m7xy = station_trans(m07[0:2], smap)
            m7.plot(ax=ax, marker='o', markersize=60, color='k')
            ax.text(m7xy[0]-10, m7xy[1]-0, 'M07', fontsize=fsize, weight=tw,
                    horizontalalignment='right', verticalalignment='center')
            m8, m8xy = station_trans(m08[0:2], smap)
            m8.plot(ax=ax, marker='o', markersize=120, color='k')
            ax.text(m8xy[0]-0, m8xy[1]+5, 'M08', color='k', fontsize=fsize, weight=tw,
                    horizontalalignment='left', verticalalignment='bottom')
            m10, m10xy = station_trans(m010[0:2], smap)
            m10.plot(ax=ax, marker='o', markersize=60, color='k')
            ax.text(m10xy[0]-3, m10xy[1]+3, 'M10', fontsize=fsize, weight=tw,
                    horizontalalignment='right', verticalalignment='bottom')
        else:
            tf, tfxy = station_trans(telfs[0:2], smap)
            tf.plot(ax=ax, marker='o', markersize=300, color='k',
                              label='Telfs', fontsize=fsize)
            ax.text(tfxy[0]+10, tfxy[1]-10, 'Telfs', fontsize=fsize, weight=tw)

    if cross:
        clabel = 'Cross section'
        for cs in cross:
            # transform and plot cross section coordinates
            crosssection = cross_trans(cs[0], cs[1], smap)
            crosssection.plot(ax=ax, marker='o', markersize=300, color='k',
                                   linestyle='-', linewidth=3,
                                   label=clabel)
            clabel=None
    if station_coords[0] and station_coords[1]:
        # transform and plot cross section coordinates
        sf, sfxy = station_trans(station_coords[0:2], smap)
        sf.plot(ax=ax, marker='.', markersize=300, color='k')
        ax.text(sfxy[0]+10, sfxy[1]+3, 'Seefeld', fontsize=fsize, weight=tw,
                horizontalalignment='left', verticalalignment='bottom')

    return ax, topo


def domains_plot(terrain, wrfin):
    '''
    plot terrain within domains and vertical levels.

    Parameters
    ----------
    terrain : list
        list containing salem datasets with terrain data for different domains
    wrfin : salem dataset
        salem dataset that contains wrfoutput data with vertical levels
    '''
    cmap = cm.get_cmap('terrain', 9)
    new_cmap = truncate_colormap(cmap, 0.225, 0.997)
    
    title=['Domain 01',
           'Domain 03',
           'Domain 04',
           'Domain 05',
           'vertical resolution']
    
    fig, axs = plt.subplot_mosaic([['a', 'a', 'a', 'a', 'b', 'b', 'b', 'b', 'b'],
                                   ['c', 'c', 'c', 'c', 'd', 'd', 'd', 'd', 'f']],
                                  figsize=[24,16],
                                  gridspec_kw={"width_ratios":[1, 1, 1, 1, 1, 1, 1, 1, 1.2]})
    axs['a'].set_facecolor('xkcd:aqua')
    terrain_plot(axs['a'], terrain[0], title=title[0], cmap=new_cmap,
                 cext=(0,3500), ter_contours=False, domain=1, cbar=False)
    con1 = ConnectionPatch(xyA=(226.1905, 151.1905), xyB=(0,0), coordsA="data", coordsB="data",
                          axesA=axs['a'], axesB=axs['b'], color="k")
    con2 = ConnectionPatch(xyA=(281.746, 151.1905), xyB=(499,0), coordsA="data", coordsB="data",
                          axesA=axs['a'], axesB=axs['b'], color="k")
    con3 = ConnectionPatch(xyA=(226.1905, 190.857), xyB=(0,370), coordsA="data", coordsB="data",
                          axesA=axs['a'], axesB=axs['b'], color="k")
    con4 = ConnectionPatch(xyA=(281.746, 190.857), xyB=(499,370), coordsA="data", coordsB="data",
                          axesA=axs['a'], axesB=axs['b'], color="k")
    axs['a'].add_artist(con1)
    axs['a'].add_artist(con2)
    axs['a'].add_artist(con3)
    axs['a'].add_artist(con4)    
    axs['a'].text(0.04, 0.95, '(a)', verticalalignment='top', 
                  transform=axs['a'].transAxes, bbox=dict(facecolor='white', alpha=0.5))
    axs['b'].text(0.04, 0.95, '(b)', verticalalignment='top',
                  transform=axs['b'].transAxes, bbox=dict(facecolor='white', alpha=0.5))
    axs['b'].set_facecolor('xkcd:aqua')
    im, topo = terrain_plot(axs['b'], terrain[1], title=title[1], cmap=new_cmap,
                   cext=(0,3500), ter_contours=False, domain=3, cbar=True)
    
    axs['c'].set_facecolor('xkcd:aqua')
    terrain_plot(axs['c'], terrain[2], title=title[2], cmap=new_cmap,station_coords=seefeld,
                 cext=(0,3500), ter_contours=True, domain=4, cbar=False)
    con1 = ConnectionPatch(xyA=(217.84, 199.02), xyB=(0,0), coordsA="data", coordsB="data",
                          axesA=axs['c'], axesB=axs['d'], color="k")
    con2 = ConnectionPatch(xyA=(272., 199.02), xyB=(500,0), coordsA="data", coordsB="data",
                          axesA=axs['c'], axesB=axs['d'], color="k")
    con3 = ConnectionPatch(xyA=(217.84, 249.23), xyB=(0,461), coordsA="data", coordsB="data",
                          axesA=axs['c'], axesB=axs['d'], color="k")
    con4 = ConnectionPatch(xyA=(272., 249.23), xyB=(500,461), coordsA="data", coordsB="data",
                          axesA=axs['c'], axesB=axs['d'], color="k")
    axs['c'].add_artist(con1)
    axs['c'].add_artist(con2)
    axs['c'].add_artist(con3)
    axs['c'].add_artist(con4)
    axs['c'].text(0.04, 0.95, '(c)', verticalalignment='top',
                  transform=axs['c'].transAxes, bbox=dict(facecolor='white', alpha=0.5))
    axs['d'].text(0.04, 0.95, '(d)', verticalalignment='top',
                  transform=axs['d'].transAxes, bbox=dict(facecolor='white', alpha=0.5))
    axs['d'].set_facecolor('xkcd:aqua')
    terrain_plot(axs['d'], terrain[3], title=title[3], cmap=new_cmap,
                 cext=(0,3500), ter_contours=True, close=True, domain=5,
                 cbar=False)
    h_agl = (wrfin.PHB + wrfin.PH)/9.8 - wrfin.HGT
    dz = np.concatenate([h_agl[0].values[..., np.newaxis],
                         (h_agl[1:] - h_agl[0:-1]).values], axis=0)
    axs['f'].grid(True)
    axs['f'].plot(dz, h_agl/1000.)
    axs['f'].set_xscale('log')
    axs['f'].set_xlabel('$\Delta z$ [m]')
    axs['f'].set_ylabel('Height a.g.l [km]')
    axs['f'].set_ylim([0, 4])
    axs['f'].set_xlim([1e0, 1e3])
    axs['f'].text(0.1, 0.97, '(e)', verticalalignment='top',
                  transform=axs['f'].transAxes, bbox=dict(facecolor='white', alpha=0.5))


    plt.savefig(outpath+'Fig1.pdf', bbox_inches="tight")


def graphical_abstract(file, **kwargs):
    '''
    Plot temperature distribution from WRFoutfput file of finest domain
    with terrain contours and wind barbs.

    Parameters
    ----------
    file : salem dataset
        salem dataset that contains the data

    Returns
    -------
    ax : axes handle
    '''
    cmap = cm.get_cmap('RdBu_r', 21)
    x = (file.west_east - file.west_east[0])/1000.
    y = (file.south_north - file.south_north[0])/1000.
    cone_fact = get_cone_factor(file.TRUELAT1, file.TRUELAT2)
    u, v = uvmet(file.U10, file.V10, file.lat, file.lon, file.STAND_LON,
             cone_fact)
    uu = ms_to_knots(u)[4::8, 4::8]
    vv = ms_to_knots(v)[4::8, 4::8]
    xm = x[4::8]
    ym = y[4::8]

    ter_range = np.arange(500, 3000, 100)
    ter_minor = ter_range[np.where(ter_range%1000!=0)]
    ter_major = ter_range[np.where(ter_range%1000==0)]

    fig, ax = plt.subplots(1, 1, figsize=(8, 7))
    c = ax.pcolor(x, y, file.TC.sel(bottom_top=0), cmap=cmap, vmin=-10., vmax=10.)
    c.set_edgecolor('face')
    fig.colorbar(c, ax=ax, label='[°C]', extend='both',
                 ticks=np.arange(-9, 10, 3))
    ax.contour(x, y, file.HGT, ter_minor, colors='dimgrey')
    contour_major = ax.contour(x, y, file.HGT, ter_major, colors='dimgrey')
    plt.clabel(contour_major, fontsize=10, inline=True, fmt = '%d m')
    ax.barbs(xm, ym, ms_to_knots(uu), ms_to_knots(vv), length=6)
    x_ticks, y_ticks = np.arange(1, x[-1]), np.arange(0, y[-1])
    x_labels = ['{} km'.format(int(num))
                for num in x_ticks]
    y_labels = ['{} km'.format(int(num))
                for num in y_ticks]
    ax.set_xticks(x_ticks)
    ax.set_xticklabels(x_labels)
    ax.set_yticks(y_ticks)
    ax.set_yticklabels(y_labels)
    plt.grid(True)
    plt.savefig('/home/andreas/Desktop/Thesis/Plots/graphical_abstract.pdf')

    return ax


#%% plot different timelines of parameters
def timeline_plot(ax, file, height, color=None, linestyle='-', linewidth=2,
                  formatter=None, title='', label='', ylabel='', xlim=[],
                  ylim=[], legend=True, **kwargs):
    '''
    Plot timelines of selected parameter at selected location.

    Parameters
    ----------
    ax : axes handle
        handle for axes to be plotted on
    file : salem dataset
        salem dataset that contains the data
    height : int
        height level for desired timeline
    color : str
        color to be used in plot. Default is black.
    linestyle : str
        linestyle to be used in plot. Default is solid line.
    linewidth : int
        width of line to be used in plot. Default is 2.
    formatter : str
        type of formatter to use in plot (color, linestyle and marker)
    title : str
        Plot title
    label : str
        Label in legend
    ylabel : str
        Label for y-axis of plot
    xlim : tuple, list
        Limits for x-axis
    ylim : tuple, list
        Limits for y-axis
    legend : bool
        Plots legend if True. Default is True.

    Returns
    -------
    ax : axis handle
    '''

    try:
        # select height level to be plotted
        timeline = file.sel(bottom_top=height)
    except KeyError:
        timeline = file

    if formatter:
        ax.plot(timeline.time, timeline, formatter, label=label,
                linestyle=linestyle, linewidth=linewidth, **kwargs)
    elif color:
        ax.plot(timeline.time, timeline, label=label,
                color=color, linestyle=linestyle, linewidth=linewidth, **kwargs) 
    else:
        ax.plot(timeline.time, timeline, label=label,
                linestyle=linestyle, linewidth=linewidth, **kwargs)
    
    ax.set_ylabel(ylabel)
    ax.set_title(title)
    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    
    if legend:
        ax.legend(mode='expand', ncol=2)

    return ax


#%% Routines to calculate and plot tendency (Figure 9)
def overview_subplot(dat_sub, ax, cap, ID='cartesian adv_form', title='',
                     **kwargs):
    '''
    plot temperature tendencies from parameterizations.

    Parameters
    ----------
    dat_sub : xr dataset
        subset of dataset with output from WRFlux postprocessing tool.
    ax : handle
        axis to plot on.
    cap : salem dataset
        defines timesteps where CAP has formed.
    ID : str
        Vertical coordinate to use. Default is 'cartesian adv_form'.
    title : str
        title of plot.
    
    Returns
    ----------
    
    '''
    try:
        dat_sub = dat_sub.sel(ID=ID)
    except KeyError:
        pass

    ax.grid(True)
    ax.set_title(title)
    ax.plot(dat_sub.time, dat_sub.rad_lw, label='LW radiation',
            color='tomato', linewidth=3, linestyle='--', **kwargs)
    ax.plot(dat_sub.time, dat_sub.mp, label='Microphysics',
            color='k', linewidth=3, linestyle='--', **kwargs)
    ax.plot(dat_sub.time, (dat_sub.adv.sel(dir='sum', comp='mean')),
            label='Advection', color='cornflowerblue', linewidth=3,
            linestyle='-', **kwargs)
    ax.plot(dat_sub.time, dat_sub.adv.sel(dir='sum', comp='trb_s'),
            label='Turb$_{sgs}$', color='cornflowerblue', linewidth=3,
            linestyle=':', **kwargs)
    ax.plot(dat_sub.time, dat_sub.adv.sel(dir='sum', comp='trb_r'),
            label='Sub-mesoscale Flow', color='cornflowerblue', linewidth=3,
            linestyle='--', **kwargs)
    ax.plot(dat_sub.time, dat_sub.net.sel(side='forcing'),
            label='Net tendency', color='k', linewidth=3, linestyle='-',
            **kwargs)
    ax.set_ylabel('$\\theta$-tendency [{}]'.format(dat_sub.units))
    ax.set_xlim(dat_sub.time[0], dat_sub.time[-1])
    ax.set_ylim([-20, 20])
    ax.legend(mode='expand', ncol=3, loc='lower center')
    ax.fill_between(cap, -20, 20, color='0.9')

def adv_tendency_subplot_comp(dat_sub, ax, cap, scale='mean', ID='cartesian adv_form',
                             title='', rotated=True, **kwargs):
    '''
    plot temperature tendencies from advection according to scale.

    Parameters
    ----------
    dat_sub : xr dataset
        subset of dataset with output from WRFlux postprocessing tool.
    ax : handle
        axis to plot on.
    cap : salem dataset
        defines timesteps where CAP has formed.
    ID : str
        Vertical coordinate to use. Default is 'cartesian adv_form'.
    scale : str
        component to be plotted ('adv_r' 'mean', 'trb_r' or 'trb_s'). Default is 'mean'.
    title : str
        title of plot.
    rotated : bool
        if True, treats values as rotated ones. Default is True.

    Returns
    ----------
    
    '''
    try:
        dat_sub = dat_sub.sel(ID=ID)
    except KeyError:
        pass

    if scale=='mean':
        limits=[-20, 20]
    else:
        limits=[-10, 10]        
    if rotated:
        xlabel, ylabel = 'Along-valley', 'Cross-valley'
        xdata, ydata = dat_sub.adv_X.sel(comp=scale), dat_sub.adv_Y.sel(comp=scale)
    else:
        xlabel, ylabel ='X', 'Y'
        xlabel, ylabel = 'Along-valley', 'Cross-valley'
        xdata, ydata = (dat_sub.adv.sel(dir='X', comp=scale),
                        dat_sub.adv.sel(dir='Y', comp=scale))
    ax.text(0.02, 0.95, '(b)', verticalalignment='top',
             transform=ax.transAxes, bbox=dict(facecolor='white', alpha=0.5))
    ax.grid(True)
    ax.set_title(title)
    ax.plot(dat_sub.time, dat_sub.adv.sel(dir='sum', comp=scale),
            label='Sum', color='k', linewidth=3, linestyle='-', **kwargs)
    ax.plot(dat_sub.time, xdata,
            label=xlabel, color='olive', linewidth=3, linestyle='-', **kwargs)
    ax.plot(dat_sub.time, ydata,
            label=ylabel, color='cornflowerblue', linewidth=3, linestyle='-',
            **kwargs)
    ax.plot(dat_sub.time, dat_sub.adv.sel(dir='Z', comp=scale),
            label='Vertical', color='tomato', linewidth=3, linestyle='-', **kwargs)
    ax.set_ylabel('$\\theta$-tendency [{}]'.format(dat_sub.units))
    ax.set_xlim(dat_sub.time[0], dat_sub.time[-1])
    ax.set_ylim(limits)
    ax.legend(mode='expand', ncol=2, loc='lower center')
    ax.fill_between(cap, -30, 30, color='0.9')


def elevation_load(dpath, sn_ind=[33,65], we_ind=[54,91]):
    '''
    load elevation from wrf output.
    
    Parameters
    ----------
    dpath : str
        path to wrf output.
    sn_ind : list/tuple
        bounding indices for south_north coordinate.
    we_ind : list/tuple
        bounding indices for west_east coordinate.

    Returns
    -------
    elevation : xarray DataArray
        DataArray with model terrain height.
    '''
    ds = salem.open_wrf_dataset(dpath)

    elevation = ds.HGT.sel(south_north=ds.south_north[sn_ind[0]:sn_ind[1]],
                           west_east=ds.west_east[we_ind[0]:we_ind[1]],
                           time=ds.time[0])

    return elevation


def prep_ds(dpath, dsm_path, elev_path, ID='cartesian adv_form', res=40,
            cp_thres=1200., sn_ind=[33,65], we_ind=[54,91],
            bottom=False, profile=False, lu_cond=True, rotation=True,
            angle_rad=0.):
    '''
    average values over cells associated with Cold Air Pool for WRFlux output.
    
    Parameters
    ----------
    dpath : str
        path to wrflux output.
    dsm_path : str
        path to mean wind output (in meanout file).
    elev_path : str
        path to wrf output including elevation information.
    ID : str
        Coordinate system of desired data. Default is 'cartesian adv_form'.
    res : int
        resolution of dataset.
    cp_thres : float
        upper elevation threshold for inclusion of cells in CAP.
    sn_ind : list/tuple
        boundaries in south_north index coordinates.
    we_ind : list/tuple
        boundaries in west_east index coordinates.
    bottom : bool
        Determines whether profiles (if False) or only bottom values (if True)
        are considered. Default is True.
    profile : bool
        Determines whether average outputs profiles (if True) or only one
        average for the whole volume (if False). Default is False.
    lu_cond : bool
        Determines whether all only grassland and pastures (if True) or all
        surfaces (if False) are considered. Default is True.
    rotation : bool
        Whether to rotate tendencies. Default is True.
    angle_rad : float
        Rotation angle in °, only needed if rotation==True. Default is 0.

    Returns
    -------
    ds_p : salem dataset
        dataset with averaged output fields over CAP coordinates.
    u_m : salem dataset
        dataset with averaged mean u wind component over CAP coordinates.
    v_m : salem dataset
        dataset with averaged mean v wind component over CAP coordinates.
    '''

    # load elevation data
    elevation = elevation_load(elev_path, sn_ind=sn_ind, we_ind=we_ind)

    # load mean wind putput
    dsm = xr.open_dataset(dsm_path)
    dsm['Time'] = timeframe
    _, index = np.unique(dsm['Time'], return_index=True)
    dsm = dsm.isel(Time=index)
    # only analyze after spinup time
    dsm = dsm.sel(Time=times)
    
    # load wrflux output
    ds = xr.open_dataset(dpath)
    _, index = np.unique(ds['Time'], return_index=True)
    ds = ds.isel(Time=index)
    ds = ds.rename({'Time': 'time'})
    # only analyze after spinup time
    ds = ds.sel(time=times)
    # subset wrflux output
    # mask all cells that are not open terrain in wrflux output
    dsl = salem.open_wrf_dataset(elev_path)

    # lowest 10 levels
    ds_cap = ds.sel(y=np.arange(ds.y[sn_ind[0]], ds.y[sn_ind[1]], res),
                    x=np.arange(ds.x[we_ind[0]], ds.x[we_ind[1]], res),
                    bottom_top=ds.bottom_top[0:10],
                    ID=ID)

    # subset mean wind output
    u_m = dsm.U_MEAN.sel(south_north=np.arange(sn_ind[0], sn_ind[1]),
                        west_east_stag=np.arange(we_ind[0], we_ind[1]),
                        bottom_top=dsm.bottom_top[0:10])
    v_m = dsm.V_MEAN.sel(south_north_stag=np.arange(sn_ind[0], sn_ind[1]),
                        west_east=np.arange(we_ind[0], we_ind[1]),
                        bottom_top=dsm.bottom_top[0:10])

    # add elevation information to wrflux output
    hgt = xr.DataArray(data=elevation.data,
                       coords=dict(y=(["y"], ds_cap.y.data),
                                   x=(["x"], ds_cap.x.data)),
                       attrs=dict(description=elevation.attrs))
    ds_cap['HGT'] = hgt
    # height above ground
    ds_cap['H_AGL'] = ds_cap.z-ds_cap.HGT
    # add layer thickness
    thick = np.concatenate([ds_cap.H_AGL.sel(bottom_top=ds_cap.bottom_top[1:]).data -
                            ds_cap.H_AGL.sel(bottom_top=ds_cap.bottom_top[0:-1]).data,
                            np.full(ds_cap.z.sel(bottom_top=ds_cap.bottom_top[0]).shape,
                                    np.nan)[:,np.newaxis,:,:]],
                           axis=1)
    ds_cap['thick'] = ds_cap.z.copy()
    ds_cap['thick'].data = thick

    # add mean wind components to wrflux output
    ds_cap['u_m'] = xr.DataArray(data=u_m.data,
                                 coords=dict(time=(["time"], ds_cap.time.data),
                                             bottom_top=(["bottom_top"], ds_cap.bottom_top.data),
                                             y=(["y"], ds_cap.y.data),
                                             x=(["x"], ds_cap.x.data)),
                                 attrs=dict(description='time-avg. x-wind component',
                                            units='m s-1'))
    ds_cap['v_m'] = xr.DataArray(data=v_m.data,
                                 coords=dict(time=(["time"], ds_cap.time.data),
                                             bottom_top=(["bottom_top"], ds_cap.bottom_top.data),
                                             y=(["y"], ds_cap.y.data),
                                             x=(["x"], ds_cap.x.data)),
                                 attrs=dict(description='time-avg. y-wind component',
                                            units='m s-1'))
    if rotation==True:
        grad_x = xr.DataArray(data=(ds_cap.adv.sel(dir='X').data/
                                    ds_cap.u_m.data),
                              coords=dict(comp=(["comp"], ds_cap.comp.data),
                                          time=(["time"], ds_cap.time.data),
                                          bottom_top=(["bottom_top"],
                                                      ds_cap.bottom_top.data),
                                          y=(["y"], ds_cap.y.data),
                                          x=(["x"], ds_cap.x.data)),
                              attrs=ds_cap.adv.sel(dir='X').attrs)
        grad_y = xr.DataArray(data=(ds_cap.adv.sel(dir='Y').data/
                                    ds_cap.v_m.data),
                              coords=dict(comp=(["comp"], ds_cap.comp.data),
                                          time=(["time"], ds_cap.time.data),
                                          bottom_top=(["bottom_top"],
                                                      ds_cap.bottom_top.data),
                                          y=(["y"], ds_cap.y.data),
                                          x=(["x"], ds_cap.x.data)),
                              attrs=ds_cap.adv.sel(dir='Y').attrs)
 
        print('Rotate tendencies and mean wind...')
        # rotate mean wind and gradient
        u_rot, v_rot = coord_trans(angle_rad, ds_cap.u_m, ds_cap.v_m)
        grad_x_rot, grad_y_rot = coord_trans(angle_rad, grad_x, grad_y)
        # getadvective tendency for x and y cordinates
        adv_X = xr.DataArray(data=(u_rot.data * grad_x_rot.data),
                             coords=dict(comp=(["comp"], grad_x_rot.comp.data),
                                         time=(["time"], grad_x_rot.time.data),
                                         bottom_top=(["bottom_top"],
                                                     grad_x_rot.bottom_top.data),
                                         y=(["y"], grad_x_rot.y.data),
                                         x=(["x"], grad_x_rot.x.data)),
                             attrs=grad_x_rot.attrs)
        adv_Y = xr.DataArray(data=v_rot.data * grad_y_rot.data,
                             coords=dict(comp=(["comp"], grad_y_rot.comp.data),
                                         time=(["time"], grad_y_rot.time.data),
                                         bottom_top=(["bottom_top"],
                                                     grad_y_rot.bottom_top.data),
                                         y=(["y"], grad_x_rot.y.data),
                                         x=(["x"], grad_x_rot.x.data)),
                             attrs=grad_y_rot.attrs)
        # add rotated tendencies to ds_cap data
        ds_cap['adv_X'] = adv_X
        ds_cap['adv_Y'] = adv_Y

    # condition to select values: everything below cp_thres terrain height
    cond = (ds_cap.z<cp_thres)

    if not bottom:
        if profile:
            dims = ['x','y']
        else:
            dims = ['x','y','bottom_top']
    else:
        dims = ['x','y']
        # only bottom level
        ds_cap = ds_cap.sel(bottom_top=ds_cap.bottom_top[0])

    if lu_cond==True:
        # calculate landuse mask: only grassland(7) and cropland(2)
        lu = ((dsl.sel(time=dsl.time[-1],
                      south_north=dsl.south_north[sn_ind[0]:sn_ind[1]],
                      west_east=dsl.west_east[we_ind[0]:we_ind[1]]).LU_INDEX==2)|
              (dsl.sel(time=dsl.time[-1],
                      south_north=dsl.south_north[sn_ind[0]:sn_ind[1]],
                      west_east=dsl.west_east[we_ind[0]:we_ind[1]]).LU_INDEX==7))
    
        ds_cap['LUmask'] = xr.DataArray(data=lu.data,
                                        coords=dict(y=(["y"], ds_cap.y.data),
                                                    x=(["x"], ds_cap.x.data)),
                                        attrs=dict(description=lu.attrs))
        # add land use to selection condition
        cond = (ds_cap.LUmask)
    if bottom==False:
        # if profile of importance, restrict also according to crest_thres
        if lu_cond==True:
            cond = ds_cap.LUmask & (ds_cap.z<cp_thres)
        else:
            cond = (ds_cap.z<cp_thres)

    # weights according to layer thickness
    weights = ((ds_cap.thick).where(cond)/
               (ds_cap.thick).where(cond).sum(dim=dims,skipna=True))

    ds_p = xr.Dataset()

    ds_p.attrs = ds_cap.attrs
    for var in list(ds_cap.keys()):
        # calculate weighted average over cells below threshold
        ds_p[var] = (ds_cap[var]*weights).sum(dim=dims,skipna=True)
        ds_p[var].attrs = ds_cap[var].attrs

    # convert to Kelvin per hour
    ds_p.attrs['units'] = ds_p.units.replace('s','h')
    for var in list(ds_p.keys()):
        ds_p[var].values = ds_p[var].values*3600.
        try:
            ds_p[var].attrs['units'] = ds_p[var].units.replace('s','h')
        except AttributeError:
            pass
    ds_p['HGT'] = ds_p.HGT/3600
    ds_p['thick'] = ds_p.thick/3600
    ds_p['H_AGL'] = ds_p.H_AGL/3600
    ds_p['z'] = (ds_cap['z']*weights).sum(dim=dims,skipna=True)
    ds_p['z'].attrs = ds_cap['z'].attrs

    return ds_p, ds_p.u_m, ds_p.v_m


def prep_wrfds(dpath, pars, sn_bounds=[33,65], we_bounds=[54,91],
               res=200, cp_thres=1200, profile=True,
               lu_cond=True):
    '''
    average values over cells associated with Cold Air Pool for WRF output.
    
    Parameters
    ----------
    dpath : str
        path to wrflux output.
    pars : list
        list of parameter names.
    sn_bounds : list/tuple
        boundaries in south_north index coordinates.
    we_bounds : list/tuple
        boundaries in west_east index coordinates.
    res : int
        resolution of dataset.
    cp_thres : float
        lower elevation threshold for inclusion of cells in CAP.
    profile : bool
        Determines whether average outputs profiles (if True) or only one
        average for the whole volume (if False). Default is True.
    lu_cond : bool
        Determines whether all only grassland and pastures (if True) or all
        surfaces (if False) are considered. Default is True.

    Returns
    -------
    ds_p : salem dataset
        dataset with averaged output fields over CAP coordinates.
    ''' 
    # load wrflux putput
    ds = salem.open_wrf_dataset(dpath)
    _, index = np.unique(ds['time'], return_index=True)
    ds = ds.isel(time=index)

    # subset wrflux output
    mask = (((ds.south_north >= ds.south_north[sn_bounds[0]]) &
             (ds.south_north < ds.south_north[sn_bounds[1]])).data[:, np.newaxis] *
            ((ds.west_east >= ds.west_east[we_bounds[0]]) &
              (ds.west_east < ds.west_east[we_bounds[1]])).data)
    ds_cap = ds.salem.subset(roi=mask)

    w = xr.Dataset(coords=dict(time=(["time"], ds_cap.time.sel(time=times).data),
                               south_north=(["south_north"], ds_cap.south_north.data),
                               west_east=(["west_east"], ds_cap.west_east.data),
                               bottom_top=(["bottom_top"], ds_cap.bottom_top.data[0:10])),
                   attrs=dict(description=ds_cap.attrs))
    w['w'] = ds_cap.W.sel(time=times)[:,0:10,:,:]
    z = ds_cap.Z.sel(time=times)[:,0:10,:,:]

    # only analyze after spinup time
    ds_cap = ds_cap.sel(time=times, bottom_top=ds.bottom_top[0:10])
    ds_cap['W'] = w['w']
    ds_cap['Z'] = z

    print('Adding Temperature [°C] to data set...')
    ds_cap['TC'] = theta_to_temp(ds_cap.THETA, ds_cap.PRESSURE)
    ds_out = xr.Dataset(coords=dict(south_north=(["south_north"], ds_cap.south_north.data),
                                    west_east=(["west_east"], ds_cap.west_east.data),
                                    bottom_top=(["bottom_top"], ds_cap.bottom_top.data)),
                          attrs=dict(description=ds_cap.attrs))
    # add elevation information to wrflux output
    ds_cap['HGT'] = ds_cap.HGT.sel(time=ds_cap.time[0])
    ds_cap['H_AGL'] = ds_cap.Z-ds_cap.HGT
    # layer thickness
    thick = np.concatenate([ds_cap.H_AGL.sel(bottom_top=ds_cap.bottom_top[1:]).data -
                            ds_cap.H_AGL.sel(bottom_top=ds_cap.bottom_top[0:-1]).data,
                            np.full(ds_cap.Z.sel(bottom_top=ds_cap.bottom_top[0]).shape,
                                    np.nan)[:,np.newaxis,:,:]],
                           axis=1)
    ds_cap['thick'] = ds_cap.Z.copy()
    ds_cap['thick'].data = thick

    print('average parameters...')
    if lu_cond:
        # mask all non-grass landuse
        lumask = ((ds.sel(time=ds.time[-1]).LU_INDEX==7)|
                  (ds.sel(time=ds.time[-1]).LU_INDEX==2))
        cond = (ds_cap.Z<cp_thres) & lumask
        cond2 = (ds_cap.Z<1500.) & (ds_cap.HGT<1193) & lumask
    else:
        cond = (ds_cap.Z<cp_thres)
        cond2 = (ds_cap.Z<1500.) & (ds_cap.HGT<1193)
    for par in pars:
        ds_out[par] = ds_cap[par].where(cond)
    tcb = ds_cap['TC'].where(cond2)
    if profile:
        dims = ['west_east','south_north']
    else:
        dims = ['west_east','south_north','bottom_top']
    # weights
    weights = ((ds_cap.thick).where(cond)/
               (ds_cap.thick).where(cond).sum(dim=dims,skipna=True))
    weights2 = ((ds_cap.thick).where(cond2)/
               (ds_cap.thick).where(cond2).sum(dim=['west_east','south_north'],skipna=True))

    ds_p = xr.Dataset()
    ds_p.attrs = ds_cap.attrs
    for var in pars:
        # calculate weighted average over cells below threshold
        ds_p[var] = (ds_cap[var]*weights).sum(dim=dims,skipna=True)
        ds_p[var].attrs = ds_cap[var].attrs
    tc = ((tcb*weights2).sum(dim=['west_east','south_north'],skipna=True))
    # calculate mean over cells below threshold
    ds_p.attrs = ds_out.attrs
    return ds_p, tc


def rot_angle(start_point, end_point):
    '''
    calculates rotation angle of cross section for given start and end points
    in equidistant coordinates (e.g. WRF coordinates in m or their indices).

    Parameters
    ----------
    start_point : list, tuple
        indices of start point
    end_point : list, tuple
        indices of end point
    Returns
    ----------
    angle : float
        rotation angle in radians
    '''
    vector_2 = np.array([end_point[0]-start_point[0],
                         end_point[1]-start_point[1]])
    vector_1 = np.array([50, 0])
    unit_vector_1 = vector_1 / np.linalg.norm(vector_1)
    unit_vector_2 = vector_2 / np.linalg.norm(vector_2)
    dot_product = np.dot(unit_vector_1, unit_vector_2)
    angle = np.arccos(dot_product)
    if start_point[1]<end_point[1]:
        angle = -angle

    return angle


def get_rot_angle(elev_path, central, lon_view, lat_view, times):
    '''
    calculate angle for rotation of coordinate system.
    
    Parameters
    ----------
    elev_path : str
        path to wrflux output.
    central : list/tuple
        coordinates of central point for subsetting dataset
        (speeds up calculation compared to full dataset).
    lon_view : int
        longitude range around central to look at.
    lat_view : int
        lattitude range around central to look at.
    times : list/array
        list of times to be analyzed.

    Returns
    -------
    angle_rad : float
        rotation angle.
    '''
    # load wrfoutput file only if not already loaded
    ds = salem.open_wrf_dataset(elev_path)
    _, index = np.unique(ds['time'], return_index=True)
    ds = ds.isel(time=index)

    print('Clipping on Tyrol region...')
    lat_start = central[0] - lat_view
    lat_end = central[0] + lat_view
    lon_start = central[1] - lon_view
    lon_end = central[1] + lon_view
    # select subset dataset
    ds_sub = ds.salem.subset(corners=((lon_start, lat_start), 
                                      (lon_end, lat_end)),
                             crs=salem.wgs84)
    # get start end end point for rotation
    end_x, end_y = loc_coords(ds_sub.sel(time=times[-1]),
                              [m04[0]+0.007, m04[1]+0.005])
    start_x, start_y = loc_coords(ds_sub.sel(time=times[-1]),
                                  [m04[0]-0.01, m04[1]-0.005])
    # calculate rotation angle
    angle_rad = rot_angle((start_x, start_y),(end_x, end_y))
    return angle_rad


def get_cap(path, central, lon_view, lat_view, times=times):
    '''
    Identify cold-air pooling episodes
    
    Parameters
    ----------
    path : str
        path to wrflux output.
    central : list/tuple
        coordinates of central point for subsetting dataset
        (speeds up calculation compared to full dataset).
    lon_view : int
        longitude range around central to look at.
    lat_view : int
        lattitude range around central to look at.
    times : list/array
        list of times to be analyzed.

    Returns
    -------
    cap : data-array
        cap times.
    '''
    # load wrfoutput file only if not already loaded
    ds = salem.open_wrf_dataset(path)
    _, index = np.unique(ds['time'], return_index=True)
    ds = ds.isel(time=index)
    # only analyze after spinup time
    ds = ds.sel(time=times)

    print('Clipping on Tyrol region...')
    lat_start = central[0] - lat_view
    lat_end = central[0] + lat_view
    lon_start = central[1] - lon_view
    lon_end = central[1] + lon_view
    # select subset dataset
    ds_sub = ds.salem.subset(corners=((lon_start, lat_start), 
                                        (lon_end, lat_end)),
                             crs=salem.wgs84)
    # define when CAP forms
    timeline = point_data(ds_sub, m04, corr=(-2, 0))
    #print('Adding WS at surface to data set...')
    #WS2wrf(timeline)
    print('Adding Temperature [°C] to data set...')
    timeline['TC'] = theta_to_temp(timeline.THETA, timeline.PRESSURE)
    cap = timeline.time.where(((timeline.TC.sel(bottom_top=3)-
                                timeline.TC.sel(bottom_top=0))>=1.8))
    return cap


def late_rotation(sf_t, u_m, v_m, thres=7, angle_rad=0, profile=False):
    '''
    locate time steps prone to cause spikes in rotated tendencies after 
    tendency calculation.
    
    '''
    grad_x = sf_t.adv.sel(dir='X')/(u_m*3600.)
    grad_y = sf_t.adv.sel(dir='Y')/(v_m*3600.)
    u_rot, v_rot = coord_trans(angle_rad, u_m, v_m)
    grad_x_rot, grad_y_rot = coord_trans(angle_rad, grad_x, grad_y)
    # getadvective tendency for x and y cordinates
    adv_X = u_rot * 3600. * grad_x_rot
    adv_X.attrs = sf_t.adv.sel(dir='X').attrs
    adv_Y = v_rot * 3600. * grad_y_rot
    adv_Y.attrs = sf_t.adv.sel(dir='Y').attrs
    # add rotated tendencies to sf_t data
    sf_t['adv_X'] = adv_X
    sf_t['adv_Y'] = adv_Y
    # detect potential spikes
    if profile==True:
        good_t = u_m.time.where(((np.abs(u_m.sel(bottom_top=u_m.bottom_top[0], 
                                                  comp='adv_r')/
                                          v_m.sel(bottom_top=u_m.bottom_top[0], 
                                                  comp='adv_r')) <= thres) |
                                  (np.abs(u_m.sel(bottom_top=u_m.bottom_top[0], 
                                                  comp='adv_r')/
                                          v_m.sel(bottom_top=u_m.bottom_top[0], 
                                                  comp='adv_r')) <= thres)),
                                drop=True)
    else:
        good_t = u_m.time.where(((np.abs(u_m / v_m) < 5) &
                                 (np.abs(v_m / u_m) < 5)),
                                drop=True)

    return sf_t, good_t


def do_fig9(outpath, epaths, dpaths, dmeanpaths, ID='cartesian adv_form'):
    """
    plotting routine for tendency plots
    """

    # times to consider for analysis
    times = pd.date_range(start='1/16/2020 11:30:00', end='1/17/2020 12:00:00', freq='10min')


    sn_ind1=[33,65]         # for 40m runs
    we_ind1=[54,91]         # for 40m runs
    res=40                  # for 40m runs
    spike_thres = 10        # threshold for spike detection
    lu_cond=False           # if True, filter for landuse class before tendency calculation

    for iname, elevation_path in enumerate(epaths):
        dspath = dpaths[iname]
        dsmean_path = dmeanpaths[iname]
        print('calculate rotation angle...')
        angle_rad = get_rot_angle(elevation_path, m04, 0.1, 0.05,
                                times=times)

        print('calculate cold pool times...')
        cap = get_cap(elevation_path, m04, 0.02, 0.01, times=times)

        thres=1200


        add_name = ""
        if "les" in dspath:
            add_name = "_les"
        print('Threshold for CAP: {} m.'.format(str(thres)))
        pars = ['TC', 'SWNORM', 'THETA', 'PRESSURE', 'W', 'CM', 'CH', 'WS', 'TSK']
        print('Calculate average parameters for 40m run...')
        ds_prep,  TC = prep_wrfds(elevation_path, pars, sn_bounds=sn_ind1, we_bounds=we_ind1,
                                        cp_thres=thres, profile=True,
                                        res=res, lu_cond=lu_cond)

        ds_prep['TC2'] = theta_to_temp(ds_prep.THETA, ds_prep.PRESSURE)
        print('Plot average surface tendencies without spike elimination...')
        print('Calculate average tendencies for 40m run...')
        sf_tend, u_mean, v_mean = prep_ds(dspath, dsmean_path, elevation_path,
                                                angle_rad=angle_rad,
                                                sn_ind=sn_ind1, we_ind=we_ind1, cp_thres=thres,
                                                bottom=False, rotation=False,
                                                res=res, lu_cond=lu_cond, profile=False)
        # find timestamp for sunset and sunrise based on incoming shortwave radiation
        sunset = ds_prep.time.where(ds_prep.SWNORM.sel(bottom_top=ds_prep.bottom_top[0])<=1, drop=True)[0].data
        sunrise = ds_prep.time.where(ds_prep.SWNORM.sel(bottom_top=ds_prep.bottom_top[0])<=1, drop=True)[-1].data
        start = sunset - pd.Timedelta(value=60, unit='min')
        end = sunrise + pd.Timedelta(value=60, unit='min')
        night_times = pd.date_range(start=start, end=end, freq='10min')
        # filter data - analysis only for nighttime
        ds_prep = ds_prep.sel(time=night_times, bottom_top=ds_prep.bottom_top[0])
        sf_tend = sf_tend.sel(time=night_times)
        TC = TC.sel(time=night_times, bottom_top=TC.bottom_top[0:8])
        u_mean = u_mean.sel(time=night_times)
        v_mean = v_mean.sel(time=night_times)
        # eliminate largest spikes
        sf_tend, good_times = late_rotation(sf_tend, u_mean, v_mean, profile=False,
                                            angle_rad=angle_rad, thres=spike_thres)
        plt.rc('font',**{'size': 23})
        fig, [ax1, ax2] = plt.subplots(1, 2, figsize=(30,10))
        overview_subplot(sf_tend, ax1, cap, title='',
                                ID=ID)
        adv_tendency_subplot_comp(sf_tend.sel(time=good_times), ax2, cap, scale='mean', ID=ID,
                                        title='',
                                        rotated=True)
        ax1.text(0.02, 0.95, '(a)', verticalalignment='top',
                transform=ax1.transAxes, bbox=dict(facecolor='white', alpha=0.5))
        ax1.text(0.02, 0.95, '(b)', verticalalignment='top',
                transform=ax2.transAxes, bbox=dict(facecolor='white', alpha=0.5))
        #plt.tight_layout()
        plt.savefig(outpath+'Fig9{}.pdf'.format(add_name))


#%% plot of map and cross section
def get_points(ter, station, wrfinner=True):
    if wrfinner:
        lat = 'XLAT'
        lon = 'XLONG'
    else:
        lat = 'XLAT_M'
        lon = 'XLONG_M'
    station_p = closest_point(list(zip(ter[lat].data.flatten(),
                            ter[lon].data.flatten())),
                            (station[0], station[1]))
    station_point = CoordPair(lon=station_p[1], lat=station_p[0])
    return station_point


def plot_cross(ax, tc_cross, theta_cross, u_cross, v_cross, xs, ys,
               w_cross, ter_line, Contourrange, 
               cbar=True):
    '''
    '''
    # levels for colorbar in plot
    levels = np.linspace(-10, 10, 21)

    tc_contours = ax.contourf(xs, ys, tc_cross, extend='both',
                              levels=levels, cmap=cm.get_cmap("RdBu_r"))
    theta_contours = ax.contour(xs, ys, to_np(theta_cross), Contourrange,
                                colors="k")
    plt.clabel(theta_contours, fontsize=10, inline=True, fmt = '%d K')
    # plot cross-wind contours
    u_contours = ax.contour(xs, ys, to_np(u_cross), np.linspace(-20, 20, 11), colors="dimgrey")

    plt.clabel(u_contours, fontsize=10, inline=True, fmt = '%d kt')

    ## plot wind barbs for cross-valley section
    ## plot wind barbs - set negative for cross section, positive for along-valley
    ax.barbs(xs[::5], ys[::5], -to_np(v_cross)[::5,::5], to_np(w_cross)[::5,::5],
         length=6)
    if cbar:
        # Add the color bar
        plt.colorbar(tc_contours, ax=ax, label='Temperature [°C]')
        #plt.colorbar(wspd_contours, ax=ax)
    
    # Fill in the mountain area
    ax.fill_between(xs, 0, to_np(ter_line),
                                    facecolor="dimgrey")
    # Set the x-ticks to use latitude and longitude labels.
    coord_pairs = to_np(theta_cross.coords["xy_loc"])
    coord_pairs = to_np(theta_cross.coords["xy_loc"])
    x_ticks = np.arange(coord_pairs.shape[0])
    x_labels = [pair.latlon_str(fmt="{:.3f}N, {:.3f}E")
                for pair in to_np(coord_pairs)]
    ax.set_xticks(x_ticks[::25])
    ax.set_xticklabels(x_labels[::25], rotation=0)
    
    # Set the y-ticks to be height.
    v_ticks = np.arange(1000,3200,200)
    ax.set_yticks(v_ticks)
    ax.set_yticklabels(v_ticks, rotation=0)
    
    # Set the x-axis and  y-axis labels
    ax.set_xlabel("Latitude, Longitude")
    ax.set_ylabel("Height [m]")
    
    
    # set limits for cross-valley section
    ax.set_ylim([1100, 2100])
    # set limits for along-valley section
    #ax.set_ylim([1100, 1700])

    # label mountains
    ax.text(0.05, 0.03,'A', horizontalalignment='center',
            verticalalignment='bottom', transform = ax.transAxes, color='white')
    ax.text(0.95, 0.03,'B', horizontalalignment='center',
            verticalalignment='bottom', transform = ax.transAxes, color='white')
    #plt.legend(loc="upper right")
    plt.close()


def plot_crossmap(filename, dsm, outpath, add_name):
    """
    Plotting function for Figure 7 of the publication.
    """
    # levels for colorbar in plot
    cext = [-10, 10]
    crosslevels = np.arange(1000,2300,10)
    Contourrange = np.arange(200, 320, 2)

    print("Loading file {}...".format(filename))
    file = Dataset(filename)
    print('Done!')

    # Define terrain
    ter = getvar(file, "ter", timeidx=0)
    cross = [((47.33, 11.162),
            (gschwandtkopf[0], gschwandtkopf[1]+.002))]
    # Define a start point and end point in grid coordinates
    start_point = get_points(ter, (47.33, 11.162))
    end_point = get_points(ter, (gschwandtkopf[0], gschwandtkopf[1]+.002))
    startloc_x, startloc_y = loc_coords_raw(ter, (47.33, 11.162))
    endloc_x, endloc_y = loc_coords_raw(ter, (gschwandtkopf[0], gschwandtkopf[1]+.002))

    angle1 = rot_angle((startloc_x, startloc_y),(endloc_x, endloc_y))

    plotrange = range(132, 145, 12)

    fig, axs = plt.subplots(2,2,figsize=(30, 20))

    axs[0,0].text(0.03, 0.95, '(a)',
            verticalalignment='top', transform=axs[0, 0].transAxes,
            bbox=dict(facecolor='white', alpha=0.9))    
    axs[0,1].text(0.03, 0.95, '(b)',
            verticalalignment='top', transform=axs[0, 1].transAxes,
            bbox=dict(facecolor='white', alpha=0.9))
    axs[1,0].text(0.03, 0.95, '(c)',
            verticalalignment='top', transform=axs[1, 0].transAxes,
            bbox=dict(facecolor='white', alpha=0.9))    
    axs[1,1].text(0.03, 0.95, '(d)',
            verticalalignment='top', transform=axs[1, 1].transAxes,
            bbox=dict(facecolor='white', alpha=0.9))
    cbar = [True] * len(plotrange)
    for j, tstep in enumerate(plotrange):
        print('Time step {}'.format(tstep))
        # Get temperature, height, wind speed and wind direction and convert to valley-oriented coordinate system.
        tc = getvar(file, "temp", timeidx=tstep, units='degC')
        z = getvar(file, "z", timeidx=tstep)
        v = getvar(file, "va", timeidx=tstep, units="kt")
        u =  getvar(file, "ua", timeidx=tstep, units="kt")
        
        uc11, vc11 = coord_trans(angle1, u, v)
        w =  getvar(file, "wa", timeidx=tstep, units="kt")
        theta = getvar(file, "th", timeidx=tstep)
        # calculate cross section
        print("Calculating cross section {}...".format(tstep))
        tc_cross = vertcross(tc, z, wrfin=file, start_point=start_point,
                                        end_point=end_point, latlon=True, meta=True,
                                        levels=crosslevels)

        u_cross = vertcross(uc11, z, wrfin=file, start_point=start_point,
                                    end_point=end_point, latlon=True, meta=True,
                                    levels=crosslevels)
        v_cross = vertcross(vc11, z, wrfin=file, start_point=start_point,
                                        end_point=end_point, latlon=True, meta=True,
                                        levels=crosslevels)
        w_cross = vertcross(w, z, wrfin=file, start_point=start_point,
                                    end_point=end_point, latlon=True, meta=True,
                                    levels=crosslevels)
        theta_cross = vertcross(theta, z, wrfin=file, start_point=start_point,
                                        end_point=end_point, latlon=True, meta=True,
                                        levels=crosslevels)

        xs = np.arange(0, tc_cross.shape[-1], 1)
        ys = to_np(tc_cross.coords["vertical"])
        # Get the terrain heights along the cross section line
        print("Calculating terrain...")
        ter_line = interpline(ter, wrfin=file, start_point=start_point,
                                        end_point=end_point)

        #plt.suptitle(str(theta_cross.Time.values).replace('T',' ')[0:19] +  ' UTC')
        plot_cross(axs[j, 1], tc_cross, theta_cross, u_cross, v_cross, xs, ys,
                w_cross, ter_line, Contourrange, cbar=cbar[j])
        level_plot(axs[j, 0], dsm.sel(time=times[tstep]), 'TC', 0, station_coords=m04, title='',
                ctitle="Temperature [°C]", cmap="RdBu_r", cross=cross, sub40=True,
                cext=cext, wind_quiver=True, cbar=True)

    fig.tight_layout()
    fig.savefig(outpath+"Fig7{}.pdf".format(add_name))


#%% Plot timelines of stations
def station_plots_all(file, station_coords, station_names, cap, rel_hum=False,
                      stationdata=False, short=False, outname='out.png',
                      **kwargs):
    '''
    plots and compares timelines of temperature, wind speed, net radiation
    and relative humidity for desired station.
    
    Parameters
    ----------
    file : salem dataset
        first dataset to be used. Should contain both WRF output and
        fields for Temperature in °C, energy balance components, etc.
    station_coords : list, sequence
        List with x, y coords of stations.
    station_names : list of strings
        List containing names of stations.
    cap : salem dataset
        defines timesteps where CAP has formed.
    rel_hum : bool
        If True, plot relative humidity instead of mixing ratio.
        Default  is False.
    stationdata : bool
        If True, plot measurement data instead of simulation data.
        Default is False.
    short : bool
        If true, plot only temperature and wind speed. Default is False.
    outname : str
        name of figure when saved.        
    '''

    tylim = (-11, 6)

    if short:
        fig, (ax1, ax2) = plt.subplots(2,1,figsize=(10,5))
        lw = 1.5
    else:
        fig, (ax1, ax2, ax3, ax4, ax5) = plt.subplots(5,1,figsize=(10,11))
        lw = 1.5
    ax1.grid(True)
    ax2.grid(True)
    ax1.text(0.02, 0.1, '(a)', verticalalignment='bottom',
             transform=ax1.transAxes, bbox=dict(facecolor='white', alpha=0.5))
    ax2.text(0.02, 0.1, '(b)', verticalalignment='bottom',
             transform=ax2.transAxes, bbox=dict(facecolor='white', alpha=0.5))
    if not short:
        ax3.grid(True)
        ax4.grid(True)
        ax5.grid(True)
        ax3.text(0.02, 0.1, '(c)', verticalalignment='bottom',
                 transform=ax3.transAxes, bbox=dict(facecolor='white', alpha=0.5))
        ax4.text(0.02, 0.1, '(d)', verticalalignment='bottom',
                 transform=ax4.transAxes, bbox=dict(facecolor='white', alpha=0.5))
        ax5.text(0.02, 0.1, '(e)', verticalalignment='bottom',
                 transform=ax5.transAxes, bbox=dict(facecolor='white', alpha=0.5))

    for i, station in enumerate(station_names):
        if station == 'M04':
            corr = (-2, 0)
        elif station == 'M10':
            corr = (1, 2)
        else:
            corr = (None, None)
        if stationdata:
            file1 = file[i]
            T = 'TAact_Avg'
            Rnet = 'NRact_Avg'
            WS = 'VWact_WVc1'
            RH = 'RHact_Avg'
            Q = 'w'
            SH = 'sh1'
            xlim = [file1.time[0], file1.time[-1]]
            ax1.plot(file1.index, file1[T], color=station_colors[i],
                     linestyle=station_linestyle[i], label=station,
                     linewidth=lw, **kwargs)
            ax1.set_ylabel('T [°C]')
            ax2.plot(file1.index, file1[WS], color=station_colors[i],
                     linestyle=station_linestyle[i], label=station, 
                     linewidth=lw, **kwargs)
            ax2.set_ylabel('WS [m s$^{-1}$]')
            if not short:
                ax3.plot(file1.index, file1[Rnet], color=station_colors[i],
                         linestyle=station_linestyle[i], label=station,
                         linewidth=lw, **kwargs)
                ax3.set_ylabel('R$_{NET}$ [W m$^{-2}$]')
                if ((station=='M04') | (station=='M08')):
                    ax4.plot(file1.index, file1[SH], color=station_colors[i],
                             linestyle=station_linestyle[i], label=station,
                             linewidth=lw, **kwargs)
                    ax4.set_ylabel('SH [W m$^{-2}$]')
                if rel_hum:
                    ax5.plot(file1.index, file1[RH], color=station_colors[i],
                             linestyle=station_linestyle[i], label=station,
                             linewidth=lw, **kwargs)
                    ax5.set_ylabel('RH [%]')
                else:
                    ax5.plot(file1.index, file1[Q]*1000, color=station_colors[i],
                             linestyle=station_linestyle[i], label=station,
                             linewidth=lw, **kwargs)
                    ax5.set_ylabel('MR [g kg$^{-1}$')
        else:
            file1 = point_data(file, station_coords[i], corr=corr)
            ax1.fill_between(cap, -120, 120, color='0.9')
            ax2.fill_between(cap, -120, 120, color='0.9')
            T = 'TC'
            Rnet = 'RNET'
            WS = 'WS'
            RH = 'RH'
            Q = 'QVAPOR'
            SH = 'HFX'
            xlim = [file1.time.data[0], file1.time.data[-1]]
            ax1 = timeline_plot(ax1, file1[T], 0, color=station_colors[i],
                                linestyle=station_linestyle[i], ylim=tylim,
                                title='', label=station, legend=False,
                                ylabel='T [°C]', surface=True,
                                xlim=xlim, linewidth=lw, **kwargs)
            ax2 = timeline_plot(ax2, file1[WS], 0, color=station_colors[i],
                                linestyle=station_linestyle[i], title='',
                                ylabel='WS [m s$^{-1}$]', label=station,
                                surface=True, ylim=[0,7.2], xlim=xlim,
                                legend=False, linewidth=lw, **kwargs)
            if not short:
                ax3 = timeline_plot(ax3, file1[Rnet], 0, color=station_colors[i],
                                    linestyle=station_linestyle[i], title='',
                                    ylim=[-100,100], xlim=xlim, label=station,
                                    surface=True, ylabel='R$_{NET}$ [W m$^{-2}$]',
                                    legend=False, linewidth=lw, **kwargs)
                ax4 = timeline_plot(ax4, file1[SH], 0, color=station_colors[i],
                                    linestyle=station_linestyle[i], title='',
                                    ylim=[-100,100], xlim=xlim, label=station,
                                    surface=True, ylabel='SH [W m$^{-2}$]',
                                    legend=False, linewidth=lw, **kwargs)
                if rel_hum:
                    ax5 = timeline_plot(ax5, file1[RH], 0, color=station_colors[i],
                                        linestyle=station_linestyle[i], 
                                        title='', label=station, 
                                        ylabel='RH [%]', surface=True, ylim=[0,102],
                                        xlim=xlim, 
                                        legend=False, linewidth=lw, **kwargs)
                else:
                    ax5 = timeline_plot(ax5, file1[Q]*1000., 0, color=station_colors[i],
                                        linestyle=station_linestyle[i], 
                                        title='', label=station,
                                        ylabel='MR [g kg$^{-1}$]', surface=True, ylim=[1,4],
                                        xlim=xlim,
                                        legend=False, linewidth=lw, **kwargs)
                ax3.fill_between(cap, -120, 120, color='0.9')
                ax4.fill_between(cap, -120, 120, color='0.9')
                ax5.fill_between(cap, -120, 120, color='0.9')
    ax1.set_yticks([-10, -5, 0, 5, 10])
    ax1.set_xlim(xlim)
    ax2.set_xlim(xlim)
    ax1.set_ylim([-11, 10])
    ax2.set_ylim([0, 5])
    if not short:
        ax3.set_ylim([-100, 100])
        ax3.set_xlim(xlim)
        ax4.set_ylim([-100, 100])
        ax4.set_xlim(xlim)
        ax5.set_xlim(xlim)
        if rel_hum:
            ax5.set_ylim([0, 102])
        else:
            ax5.set_ylim([1, 4])
    handles, lbls = ax1.get_legend_handles_labels()
    lgd = fig.legend(handles, lbls, bbox_to_anchor=(0.5, -0.05),
                     loc='lower center', ncol=6)
    plt.tight_layout()
    plt.savefig(outpath+outname, bbox_extra_artists=(lgd,), bbox_inches='tight')
    plt.close()


def station_plots_both(file, file2, station_coords, station_names, cap, rel_hum=False,
                      short=False, outname='out.png',
                      **kwargs):
    '''
    plots differences between measured and simulated temperature, wind speed,
    net radiation, and relative humidity for desired station.
    
    Parameters
    ----------
    file : salem dataset
        first dataset to be used. Should contain both WRF output and
        fields for Temperature in °C, energy balance components, etc.
    file2 : list of pandas dataframes
        second dataset to be used. Should contain both measurement results.
    station_coords : list, sequence
        List with x, y coords of stations.
    station_names : list of tuples
        List containing names of stations and a qualitative description of
        their respective location.
    cap : salem dataset
        defines timesteps where CAP has formed.
    rel_hum : bool
        If True, plot relative humidity instead of mixing ratio.
        Default  is False.
    short : bool
        If true, plot only temperature and wind speed. Default is False.
    outname : str
        name of figure when saved.        
    '''

    if short:
        fig, (ax1, ax2) = plt.subplots(2,1,figsize=(10,5))
        lw = 1.5
    else:
        fig, (ax1, ax2, ax3, ax4, ax5) = plt.subplots(5,1,figsize=(10,11))
        lw = 1.5
    ax1.grid(True)
    ax2.grid(True)
    ax1.text(0.02, 0.1, '(a)', verticalalignment='bottom',
             transform=ax1.transAxes, bbox=dict(facecolor='white', alpha=0.5))
    ax2.text(0.02, 0.1, '(b)', verticalalignment='bottom',
             transform=ax2.transAxes, bbox=dict(facecolor='white', alpha=0.5))
    #ax1.plot((), (-15, 15)'k-', linewidth=0.5)
    if not short:
        ax3.grid(True)
        ax4.grid(True)
        ax5.grid(True)
        ax3.text(0.02, 0.1, '(c)', verticalalignment='bottom',
                 transform=ax3.transAxes, bbox=dict(facecolor='white', alpha=0.5))
        ax4.text(0.02, 0.1, '(d)', verticalalignment='bottom',
                 transform=ax4.transAxes, bbox=dict(facecolor='white', alpha=0.5))
        ax5.text(0.02, 0.1, '(e)', verticalalignment='bottom',
                 transform=ax5.transAxes, bbox=dict(facecolor='white', alpha=0.5))

    T_st = 'TAact_Avg'
    Rnet_st = 'NRact_Avg'
    WS_st = 'VWact_WVc1'
    RH_st = 'RHact_Avg'
    Q_st = 'w'
    SH_st = 'sh1'
    T = 'TC'
    Rnet = 'RNET'
    WS = 'WS'
    RH = 'RH'
    Q = 'QVAPOR'
    SH = 'HFX'

    for i, station in enumerate(station_names):
        if station[0] == 'M04':
            corr = (-2, 0)
        elif station[0] == 'M10':
            corr = (1, 2)
        else:
            corr = (None, None)
        
        file1 = point_data(file.sel(bottom_top=0), station_coords[i], corr=corr)
        file_st =  file2[i]

        xlim = [file1.time.data[0], file1.time.data[-1]]
        ax1.plot(file_st.time, file_st[T_st], color=station_colors2[i],
                 linestyle='--', label='{} ({})'.format(station[0], station[1]),
                 linewidth=lw, **kwargs)
        ax1.plot(file1.time, file1[T], color=station_colors2[i],
                 linestyle='-', label=station[0]+'$_{sim}$ '+'({})'.format(station[1]),
                 linewidth=lw, **kwargs)
        ax1.set_ylabel('T [°C]')
        ax2.plot(file_st.time, file_st[WS_st], color=station_colors2[i],
                 linestyle='--', label='{} ({})'.format(station[0], station[1]),
                 linewidth=lw, **kwargs)
        ax2.plot(file1.time, file1[WS], color=station_colors2[i],
                 linestyle='-', label=station[0]+'$_{sim}$ '+'({})'.format(station[1]), 
                 linewidth=lw, **kwargs)
        ax2.set_ylabel('WS [m s$^{-1}$]')
        if not short:
            ax3.plot(file_st.time, file_st[Rnet_st], color=station_colors2[i],
                     linestyle='--', label='{} ({})'.format(station[0], station[1]),
                     linewidth=lw, **kwargs)
            ax3.plot(file1.time, file1[Rnet], color=station_colors2[i],
                     linestyle='-', label=station[0]+'$_{sim}$ '+'({})'.format(station[1]),
                     linewidth=lw, **kwargs)
            ax3.set_ylabel('R$_{NET}$ [W m$^{-2}$]')
            if ((station[0]=='M04') | (station[0]=='M08')):
                ax4.plot(file_st.time, file_st[SH_st], color=station_colors2[i],
                         linestyle='--', label='{} ({})'.format(station[0], station[1]),
                         linewidth=lw, **kwargs)
                ax4.plot(file1.time, file1[SH], color=station_colors2[i],
                         linestyle='-', label=station[0]+'$_{sim}$ '+'({})'.format(station[1]),
                         linewidth=lw, **kwargs)
                ax4.set_ylabel('SH [W m$^{-2}$]')
            if rel_hum:
                ax5.plot(file_st.time, file_st[RH_st], color=station_colors2[i],
                         linestyle='--', label='{} ({})'.format(station[0], station[1]),
                         linewidth=lw, **kwargs)
                ax5.plot(file1.time, file1[RH], color=station_colors2[i],
                         linestyle='-', label=station[0]+'$_{sim}$ '+'({})'.format(station[1]),
                         linewidth=lw, **kwargs)
                ax5.set_ylabel('RH [%]')
            else:
                ax5.plot(file_st.time, file_st[Q_st]*1000., color=station_colors2[i],
                         linestyle='--', label='{} ({})'.format(station[0], station[1]),
                         linewidth=lw, **kwargs)
                ax5.plot(file1.time, (file1[Q])*1000, color=station_colors2[i],
                         linestyle='-', label=station[0]+'$_{sim}$ '+'({})'.format(station[1]),
                         linewidth=lw, **kwargs)
                ax5.set_ylabel('MR [g kg$^{-1}$')
    #ax2.legend(loc='upper center', mode='expand', ncol=6)
    ax1.set_yticks([-10, -5, 0, 5, 10])
    ax1.set_xlim(xlim)
    ax2.set_xlim(xlim)
    ax1.set_ylim([-11, 10])
    ax2.set_ylim([0, 5])
    ax1.set_yticks([-10, -5, 0, 5, 10])
    ax1.fill_between(cap, -120, 120, color='0.9')
    ax2.fill_between(cap, -120, 120, color='0.9')
    if not short:
        ax3.fill_between(cap, -120, 120, color='0.9')
        ax4.fill_between(cap, -120, 120, color='0.9')
        ax5.fill_between(cap, -120, 120, color='0.9')
        ax3.set_ylim([-100, 100])
        ax3.set_xlim(xlim)
        ax4.set_ylim([-100, 100])
        ax4.set_xlim(xlim)
        ax5.set_xlim(xlim)
        if rel_hum:
            ax5.set_ylim([0, 102])
        else:
            ax5.set_ylim([1, 4])
    handles, lbls = ax1.get_legend_handles_labels()
    lgd = fig.legend(handles, lbls, bbox_to_anchor=(0.5, -0.05), loc='lower center', ncol=3)
    plt.tight_layout()
    plt.savefig(outpath+outname, bbox_extra_artists=(lgd,), bbox_inches='tight')
    plt.close()


def les_paperplot(file, file2, station_coords, station_names, cap,
                  outname='out.png', **kwargs):
    '''
    plots differences between measured and simulated temperature, wind speed,
    net radiation, and relative humidity for desired station.
    
    Parameters
    ----------
    file : salem dataset
        first dataset to be used. Should contain both WRF output and
        fields for Temperature in °C, energy balance components, etc.
    file2 : list of pandas dataframes
        second dataset to be used. Should contain both measurement results.
    station_coords : list, sequence
        List with x, y coords of stations.
    station_names : list of strings
        List containing names of stations.
    cap : salem dataset
        defines timesteps where CAP has formed.
    outname : str
        name of figure when saved.        
    '''

    fig, (ax1, ax2) = plt.subplots(2,1,figsize=(10,4.))
    lw = 1.5

    ax1.grid(True)
    ax2.grid(True)
    ax1.text(0.02, 0.1, '(a)', verticalalignment='bottom',
             transform=ax1.transAxes, bbox=dict(facecolor='white', alpha=0.5))
    ax2.text(0.02, 0.1, '(b)', verticalalignment='bottom',
             transform=ax2.transAxes, bbox=dict(facecolor='white', alpha=0.5))

    T_st = 'TAact_Avg'
    WS_st = 'VWact_WVc1'
    T = 'TC'
    WS = 'WS'

    for i, station in enumerate(station_names):
        if station[0] == 'M04':
            corr = (-2, 0)
        elif station[0] == 'M10':
            corr = (1, 2)
        else:
            corr = (None, None)
        
        file1 = point_data(file.sel(bottom_top=0), station_coords[i], corr=corr)
        file_st =  file2[i]

        xlim = [file1.time.data[0], file1.time.data[-1]]
        ax1.plot(file_st.time, file_st[T_st], color=station_colors2[i],
                 linestyle='--', label=station[0],
                 linewidth=lw, **kwargs)
        ax1.plot(file1.time, file1[T], color=station_colors2[i],
                 linestyle='-', label=station[0]+'$_{sim}$'+' ({})'.format(station[1]),
                 linewidth=lw, **kwargs)
        ax1.set_ylabel('T [°C]')
        ax2.plot(file_st.time, file_st[WS_st], color=station_colors2[i],
                 linestyle='--', label=station[0],
                 linewidth=lw, **kwargs)
        ax2.plot(file1.time, file1[WS], color=station_colors2[i],
                 linestyle='-', label=station[0]+'$_{sim}$'+' ({})'.format(station[1]), 
                 linewidth=lw, **kwargs)
        ax2.set_ylabel('WS [m s$^{-1}$]')
    ax1.set_yticks([-10, -5, 0, 5, 10])
    ax1.set_xlim(xlim)
    ax2.set_xlim(xlim)
    ax1.set_ylim([-11, 10])
    ax2.set_ylim([0, 5])
    ax1.fill_between(cap, -120, 120, color='0.9')
    ax2.fill_between(cap, -120, 120, color='0.9')
    handles, lbls = ax1.get_legend_handles_labels()
    lgd = fig.legend(handles, lbls, bbox_to_anchor=(0.5, -0.05), loc='upper center', ncol=3)
    #fig.tight_layout()
    plt.savefig(outpath+outname, bbox_extra_artists=(lgd,), bbox_inches='tight')
    plt.close()


#%% scatter plot of cells within basin
def pseudo_scatter_mult(ds, par, sn, we, refdata, refheight,
                        trange, height=0, bench=0., add_name=""):
    '''
    Plot scatter plots of parameter par in dependence of height hpar 
    for time steps trange in one figure.

    Parameters
    ----------
    ds : salem dataset
        Dataset containing WRF output.
    par : str
        Name of parameter.
    sn : list/tuple
        Range of cells (as indices) that are considered in the plot
        in south_north direction.
    we : list/tuple
        Range of cells (as indices) that are considered in the plot
        in west_east direction.
    refdata : pd dataframe
        Measured pseudoprofile data.
    refheight : array
        Height of measurement stations.
    trange : str
        Time steps at which plots are created.
    height : int
        Height level to be plotted. Default is 0 (lowest model level).
    bench : float
        Benchmark wind speed to indicate if surpassed (in m/s). Default is 0.
    add_name : str
        String to be added to name of figure. Default is "".
    '''
    if par=='T2C':
        wmask = ds.WS.sel(bottom_top=ds.bottom_top[height]) < bench
        hpar = 'HGT'
        ylim = [1170, 1270]
    else:
        wmask = ds.WS.sel(bottom_top=ds.bottom_top[height]) < bench
        ds = ds.sel(bottom_top=ds.bottom_top[height])
        hpar = 'Z'
        ylim = [1180, 1280]

    south, north = sn[0], sn[1]
    west, east = we[0], we[1]

    ds = ds.sel(south_north=ds.south_north[south:north],
                west_east=ds.west_east[west:east])
    fig, axs = plt.subplots(1,3,figsize=(10,3.5))
    for i, tstep in enumerate(trange):
        # seperate profile into MOMAAS and HOBOS
        momaas = refdata[m_names][refdata.index==ds.time[tstep].data].T
        hobos = refdata[h_names][refdata.index==ds.time[tstep].data].T
        # also seperate heights according to station type
        heights_h = refheight[[1, 2, 4, 5, 6, 8, 9, 10, 11]]
        heights_m = refheight[[0, 3, 7, 12]]
        axs.flatten()[i].grid(True)
        axs.flatten()[i].plot(hobos, heights_h, color='0.2', linestyle='-',
                              label='HOBOs (meas.)', marker='o',
                              markersize=6, linewidth=2)
        axs.flatten()[i].plot(momaas, heights_m, color='0.2', linestyle=None,
                              marker='X', markersize=6, linewidth=0,
                              label='Rotronics (meas.)')
        dsp = ds.sel(time=ds.time[tstep])
        dsp['wmask'] = wmask.sel(time=ds.time[tstep])
        print(dsp[hpar].min())
        print(dsp[hpar].max())
        for il, luc in enumerate(lu_class):
            dsp['lmask'] = ((dsp.LU_INDEX==luc[0])|(dsp.LU_INDEX==luc[1]))
            axs.flatten()[i].scatter(dsp[par].where(dsp.lmask&(dsp.wmask)).data.flatten(),
                       dsp[hpar].where(dsp.lmask&(dsp.wmask)).data.flatten(),
                       marker='o', facecolors=colors[il], label=labels[il], s=10)
            axs.flatten()[i].scatter(dsp[par].where(dsp.lmask&(~dsp.wmask)).data.flatten(),
                       dsp[hpar].where(dsp.lmask&(~dsp.wmask)).data.flatten(),
                       marker='o', facecolors=colors[il], edgecolors='k', s=10)

        axs.flatten()[i].set_ylim(ylim)
        axs.flatten()[i].set_yticks(np.arange(1180,1281,20),
                                    [1180, 1200, 1220, 1240, 1260, 1280])
        axs.flatten()[i].set_xlim((-10.5, 3))
        axs.flatten()[i].set_xlabel('Temperature [°C]')
        axs.flatten()[i].set_ylabel('Elevation [m a.s.l.]')
        axs.flatten()[i].text(0.025, 0.974, subplot_labels[i],
                              verticalalignment='top',
                             transform=axs.flatten()[i].transAxes,
                             bbox=dict(facecolor='white', alpha=0.9))

    handles, lbls = axs.flatten()[1].get_legend_handles_labels()
    lgd = fig.legend(handles, lbls, bbox_to_anchor=(0.5, -0.1), loc='lower center', ncol=5)
    plt.tight_layout()
    fig.savefig(outpath+"Fig8{}.pdf".format(add_name),
                bbox_extra_artists=(lgd,), bbox_inches='tight')
    plt.close()


#%% plot of pseudo profiles
def pseudo_profile(ds, heights, trange, text, colors, add_name=""):
    '''
    Plot Pseudo profiles of parameter par in dependence of height hpar 
    for time steps trange in one figure.

    Parameters
    ----------
    ds : salem dataset, pd dataframe
        Dataset containing WRF output or measurement results.
    heights : np.array
        Station heights for each station.
    trange : str
        Time steps at which plots are created.
    text : str
        Label within plot.
    colors : list
        List of colors for different time steps.
    add_name : str
        String to be added to name of figure. Default is "".
    '''
    #plt.rc('font',**{'size': 13})
    
    ylim = [1180, 1270]
    # seperate profile into MOMAAS and HOBOS
    momaas = pd.DataFrame(data=ds[m_names].values.T,
                      index=ds[m_names].columns,
                      columns=ds[m_names].index)
    hobos = pd.DataFrame(data=ds[h_names].values.T,
                      index=ds[h_names].columns,
                      columns=ds[h_names].index)
    # also seperate heights according to station type
    heights_h = heights[[1, 2, 4, 5, 6, 8, 9, 10, 11]]
    heights_m = heights[[0, 3, 7, 12]]

    fig, ax = plt.subplots(1,1,figsize=(5,7.5))
    for i, tstep in enumerate(trange):
        #fig, ax = plt.subplots(1,1,figsize=(10,15))
        plt.grid(True)
        nametime = str(momaas.columns[tstep]).replace('T', ' ')[5:16]
        ax.plot(hobos[hobos.columns[tstep]], heights_h, label=nametime,
                linestyle='-', color=colors[i], marker='o', markersize=7)

        for i_s, ind in enumerate(momaas.index):
            ax.plot(momaas[momaas.columns[tstep]][ind], heights_m[i_s],
                    marker=markers[i_s], linestyle=None, color=colors[i],
                    markersize=7)

        ax.set_ylim(ylim)
        ax.set_xlim((-11, 4))
        ax.text(0.91, 0.97, text, verticalalignment='top',
                transform=ax.transAxes,
                bbox=dict(facecolor='white', alpha=0.9))
        ax.set_xlabel('Temperature [°C]')
        ax.set_ylabel('Elevation [m a.s.l.]')
    plt.legend(loc='upper left')
    plt.tight_layout()
    fig.savefig(outpath+"Fig5_{}{}.pdf".format(text[1:-1], add_name))
    plt.close()
